@@ -20,6 +20,12 @@
 /* Defined when your system has a define or typedef for time_t. */
 #undef HAVE_TYPE_TIME_T
 
+/* Defined when the system has a working dlopen () call. */
+#undef HAVE_DLOPEN
+
+/* Defined when the system has a working shl_load () call instead. */
+#undef USE_SHL_LOAD
+
 @BOTTOM@
 #if defined (Solaris)
 #  include <arch/solaris/prototypes.h>
