@@ -178,7 +178,11 @@ split_line (char *line, int at_character)
       end = 0;
       for (i = start; line[i]; i++)
 	{
-	  if (quoted && (line[i] == '"'))
+	  if (quoted && (line[i] == '\\'))
+	    {
+	      i++;
+	    }
+	  else if (quoted && (line[i] == '"'))
 	    {
 	      end = i;
 	      i++;
