@@ -33,8 +33,8 @@
 		 (copy-syntax-table lisp-mode-syntax-table))
 	   (modify-syntax-entry ?\< "(>" mhtml-mode-syntax-table)
 	   (modify-syntax-entry ?\> ")<" mhtml-mode-syntax-table)
-	   (modify-syntax-entry ?\( "-" mhtml-mode-syntax-table)
-	   (modify-syntax-entry ?\) "-" mhtml-mode-syntax-table)
+	   ;;; (modify-syntax-entry ?\( "-" mhtml-mode-syntax-table)
+	   ;;; (modify-syntax-entry ?\) "-" mhtml-mode-syntax-table)
 	   (modify-syntax-entry ?\; "w" mhtml-mode-syntax-table)
 	   (modify-syntax-entry ?\& "w" mhtml-mode-syntax-table)
 	   (modify-syntax-entry ?\| "w" mhtml-mode-syntax-table)))
@@ -60,7 +60,7 @@
   (make-local-variable 'comment-start)
   (setq comment-start ";;; ")
   (make-local-variable 'comment-start-skip)
-  (setq comment-start-skip ";;;+ *")
+  (setq comment-start-skip ";;+ *")
   (make-local-variable 'comment-column)
   (setq comment-column 40)
   (make-local-variable 'comment-indent-function)
@@ -88,7 +88,8 @@
   '("set-var" "get-var" "unset-var" "subst-in-var" "subst-in-string"
     "package-vars" "package-names" "package-to-alist" "alist-to-package"
     "increment" "if" "ifeq" "when" "not" "and" "or" "var-case" "while"
-    "defmacro" "defsubst" "defun" "debugging-on" "match" "include"
+    "defmacro" "defsubst" "defun" "define-container" "define-function"
+    "define-tag" "define-variable" "defvar" "debugging-on" "match" "include"
     "replace-page" "redirect" "cgi-encode" "cgi-decode" "cgi-exec" "comment"
     "verbatim" "plain-text" "helper" "input-item" "small-caps"
     "html-quote" "subst-in-page" "set-form-input-values" "page-insert"
@@ -104,6 +105,16 @@
     "in-package" "with-local-package" "foreach" "array-size" "array-append"
     "array-add-unique" "layout::page" "layout::left-border"
     "layout::right-border"
+    "sql::with-open-database" "sql::database-exec-query"
+    "sql::database-exec-sql" "sql::database-next-record"
+    "sql::database-save-record" "sql::database-delete-record"
+    "sql::database-load-record" "sql::database-save-package"
+    "sql::number-of-rows" "sql::set-row-position" "sql::database-query"
+    "sql::host-databases" "sql::database-tables" "sql::database-tables-info"
+    "sql::database-table-info" "sql::database-columns"
+    "sql::database-column-info" "sql::database-columns-info"
+    "sql::database-query-info" "sql::database-set-options"
+    "sql::cursor-get-column" "sql::query-get-column" "sql::sql-transact"
     "example" "simple-fundoc" "complex-fundoc" "prog" "server-push"))
 
 (defun mhtml-build-font-lock-keywords (from-list)
@@ -134,6 +145,8 @@ if that value is non-nil."
   (kill-all-local-variables)
   (make-local-variable 'font-lock-keywords)
   (setq font-lock-keywords mhtml-font-lock-keywords)
+  ;;; (make-local-variable 'font-lock-comment-start-regexp)
+  (setq font-lock-comment-start-regexp ";;+")
   (use-local-map mhtml-mode-map)
   (set-syntax-table mhtml-mode-syntax-table)
   (setq major-mode 'mhtml-mode)
@@ -203,7 +216,7 @@ rigidly along with this one."
 
 (defconst mhtml-complex-tags
   '( "when" "while" "with" "defsubst" "defun" "defmacro" "defstruct"
-     "define-tag" "define-function"
+     "define-tag" "define-function" "with-open-dmaild"
      "with-open-database" "with-open-stream" "server-push"
      "comment" "verbatim" "small-caps" "plain-text" "ptext" "section"
      "ul" "ol" "dl" "table" "tr" "td" "center" "a" "form" "select" "caption"
@@ -212,7 +225,8 @@ rigidly along with this one."
      "menu" "system-menu" "foreach" "menu" "system-menu"
      "layout::page" "layout::left-border" "layout::right-border"
      "frameset" "msql::with-open-database" "odbc::with-open-database"
-     "dbm::with-open-database" "def-pagelist")
+     "sql::with-open-database" "dbm::with-open-database" "def-pagelist"
+     "defweakmacro" "with-safe-environment")
   "List of tag starts which are complex.")
 
 (defconst mhtml-complex-tag-start-regex
