@@ -680,6 +680,8 @@ engine_make_request (void)
     temp = (char *)getenv ("METAHTML_URL_COOKIE");
     if (temp == (char *)NULL)
       temp = (char *)getenv ("HTTP_METAHTML_URL_COOKIE");
+    if (temp == (char *)NULL)
+      temp = (char *)getenv ("REDIRECT_METAHTML_URL_COOKIE");
     if (temp != (char *)NULL)
       bprintf (headers, "URL-Cookie: SID=%s\n", temp);
 
@@ -748,7 +750,8 @@ process_request (void)
   pagefunc_set_variable ("mhttpd::requester", req->requester);
   pagefunc_set_variable ("mhttpd::requester-addr", req->requester_addr);
   pagefunc_set_variable ("mhttpd::virtual-host",
-			 (char *)getenv ("HTTP_HOSTNAME"));
+			 (char *)getenv ("HTTP_HOST"));
+  mhtml_evaluate_string ("<subst-in-var mhttpd::virtual-host \":.*$\" \"\">");
 
   if (req && mhttpd_per_request_function)
     {
@@ -795,7 +798,7 @@ process_request (void)
 	      }
 	}
 
-      fwrite (result->page->buffer + i, result->page->bindex, 1, stdout);
+      fwrite (result->page->buffer + i, 1, result->page->bindex - i, stdout);
       fflush (stdout);
     }
   if (result)
