@@ -54,10 +54,25 @@ static PFunDesc func_table[] =
 };
 
 PACKAGE_INITIALIZER (initialize_array_functions)
+DOC_SECTION (ARRAYS)
 
-/* <array-size foo> --> number of allocated elements in FOO. */
-static void
-pf_array_size (PFunArgs)
+DEFUN (pf_array_size, arrayvar,
+"Returns the number of elements in the array referenced by the
+variable <var arrayvar>.
+
+Examples:
+
+<complete-example>
+<set-var array[]=\"this\">
+<array-size array>
+</complete-example>
+
+and,
+
+<complete-example>
+<array-shift 4 array>
+<array-size array>
+</complete-example>")
 {
   char *array_name = mhtml_evaluate_string (get_positional_arg (vars, 0));
   int result = 0;
@@ -74,10 +89,26 @@ pf_array_size (PFunArgs)
   xfree (array_name);
 }
 
-/* <array-member item array [caseless=true]>
-       --> element-offset of item in array or "". */
-static void
-pf_array_member (PFunArgs)
+DEFUN (pf_array_member, item arrayvar &key caseless=true,
+"Look up (and return) the index of <var item> in the contents of the
+array referenced by <var arrayvar>.
+
+If <var item> is not found, then <code>array-member</code> returns the
+empty string.
+
+If <var caseless> is non-empty, then the comparison is done without
+regard to character case.  Otherwise, character case is significant in
+the location of the item.
+
+<complete-example>
+<set-var array[] =
+  <prog
+     this
+     another
+     multi word
+     thing>>
+<array-member \"multi word\" array>
+</complete-example>")
 {
   char *item = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *array = mhtml_evaluate_string (get_positional_arg (vars, 1));
@@ -116,9 +147,19 @@ pf_array_member (PFunArgs)
   xfree (array);
 }
 
-/* <array-append item arrayvar> --> add ITEM to the end of ARRAYVAR. */
-static void
-pf_array_append (PFunArgs)
+DEFUN (pf_array_append, item arrayvar,
+"Add <var item> as the last array element of the contents
+of <var arrayvar>.  This is especially useful in conjunction with
+<funref arrays foreach> as a <i>collector</i>:
+
+<example>
+<foreach name allnames>
+  <if <satifies-criteria <get-var name>>
+     <array-append <get-var name> useful-names>>
+</foreach>
+</example>
+
+See also <funref arrays array-add-unique>.")
 {
   char *item = mhtml_evaluate_string (get_positional_arg (vars, 0)); 
   char *array = mhtml_evaluate_string (get_positional_arg (vars, 1));
@@ -135,10 +176,15 @@ pf_array_append (PFunArgs)
   xfree (item);
 }
 
-/* <array-add-unique item arrayvar [caseless=true]>
-       --> add ITEM to ARRAYVAR if not already present. */
-static void
-pf_array_add_unique (PFunArgs)
+DEFUN (pf_array_add_unique, item arrayvar &key caseless=true,
+"Add <var item> as the last array element of the contents
+of <var arrayvar> if, and only if, <var item> is not already
+a member of that array.
+
+The comparison is a direct string-wise compare.  If <var CASELESS> is
+non-empty, then a caseless string compare is done.
+
+See also <funref arrays array-append>.")
 {
   char *item = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *array = mhtml_evaluate_string (get_positional_arg (vars, 1));
@@ -180,14 +226,41 @@ pf_array_add_unique (PFunArgs)
   xfree (item);
 }
 
-/* <array-shift amount arrayvar [start=0]>
-   Shift the elements of ARRAYVAR the indicated amount.
-   If AMOUNT is negative, the elements are shifted down, with the
-   lowest number elements being lost.
-   If AMOUNT is positive, the elements are shifted up, with no loss at
-   all. */
-static void
-pf_array_shift (PFunArgs)
+DEFUN (pf_array_shift, amount arrayvar,
+"Shift the elements of <var arrayvar> the indicated amount.
+
+If <var amount> is negative, the elements are shifted down (i.e. towards zero), with the lowest number elements being lost.
+
+If <var amount> is positive, the elements are shifted up, with no loss at all -- instead empty elements are used to fill the created space.
+
+Given the array:
+<example>
+<set-var array[] =
+   <prog
+       0
+       1
+       2>>
+</example>
+
+<set-var array[] =
+   <prog
+       0
+       1
+       2>>
+
+then after executing <example code><array-shift 2 array></example>, the array looks like:
+<example>
+   \"\"
+   \"\"
+   \"0\"
+   \"1\"
+   \"2\"
+</example>
+and, a subsequent execution of <example code><array-shift -3 array></example> leaves <var array>:
+<example>
+   \"1\"
+   \"2\"
+</example>")
 {
   char *amount_txt =  mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *array_var = mhtml_evaluate_string (get_positional_arg (vars, 1));
@@ -264,12 +337,34 @@ pf_array_shift (PFunArgs)
   xfree (start_var);
 }
 
-/* <foreach elementvar arrayvar [start=X] [end=X]> body </foreach>
-   Perform body with ELEMENTVAR bound to successive memebers of
-   ARRAYVAR, starting with the element at START (default 0), and
-   ending at END (default <array-size ARRAYVAR>). */
-static void
-pf_foreach (PFunArgs)
+DEFMACRO (pf_foreach, elementvar arrayvar &key start=x end=x step=x,
+"Perform <var body> with <var elementvar> bound to successive memebers
+of <var arrayvar>, starting with the element at <var start> (default
+0), and ending at <var end> (default <example code><array-size
+ARRAYVAR></example>), advancing by <var step> (default 1).
+
+The <code>foreach</code> command is the basic array looping device in
+<Meta-HTML>.  It is guaranteed to iterate over each element that you
+specify, whether that element is the empty string or not.
+
+Starting with the simple array:
+<set-var array[]=\"0\\n1\\n2\\n3\\n4\\n5\\n6\\n7\\n8\\n9\">
+<example>
+<set-var array[]=\"0\\n1\\n2\\n3\\n4\\n5\\n6\\n7\\n8\\n9\">
+</example>
+
+we can print out the odd numbers of this array by using values for
+both <var start> and <var step>:
+
+<complete-example>
+<foreach x array start=1 step=2> <get-var x>, </foreach>
+</complete-example>
+
+or, we can produce a \"countdown\" with a negative value for <var step>:
+
+<complete-example>
+<foreach x array step=-1> <get-var x>, </foreach> BOOM!
+</complete-example>")
 {
   char *element_var = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *array_var = mhtml_evaluate_string (get_positional_arg (vars, 1));
@@ -372,10 +467,23 @@ pf_foreach (PFunArgs)
   xfree (step_arg);
 }
 
-/* <array-concat receiver contributor...> append CONTRIBUTORs to the end of 
-   RECEIVER. */
-static void
-pf_array_concat (PFunArgs)
+DEFUN (pf_array_concat, receiver &optional contributor...,
+"Appends the contents of each <var contributor> array to the 
+end of <var receiver>.
+
+Both <var receiver> and each <var contributor> are variable names whose
+values are treated as arrays.
+
+For a single <var contributor>, <code>array-concat</code>
+could have been defined as:
+
+<example>
+<defsubst array-concat>
+  <foreach item %1>
+     <array-append <get-var item> %0>
+  </foreach>
+</defsubst>
+</example>")
 {
   char *arrayname = mhtml_evaluate_string (get_positional_arg (vars, 0));
 
@@ -441,12 +549,21 @@ sort_with_function (const void *item1, const void *item2)
     {
       PAGE *page = page_create_page ();
 
-      bprintf (page, "<%s %s>", mhtml_sort_function_name, string1);
-      string1 = mhtml_evaluate_string (page->buffer);
+      if (string1)
+	{
+	  char *temp = quote_for_setvar (string1);
+	  bprintf (page, "<%s %s>", mhtml_sort_function_name, temp);
+	  string1 = mhtml_evaluate_string (page->buffer);
+	}
       page->bindex = 0;
-      bprintf (page, "<%s %s>", mhtml_sort_function_name, string2);
-      page->buffer[page->bindex] = '\0';
-      string2 = mhtml_evaluate_string (page->buffer);
+
+      if (string2)
+	{
+	  char *temp = quote_for_setvar (string2);
+	  bprintf (page, "<%s %s>", mhtml_sort_function_name, temp);
+	  page->buffer[page->bindex] = '\0';
+	  string2 = mhtml_evaluate_string (page->buffer);
+	}
 
       page_free_page (page);
       should_free++;
@@ -483,9 +600,88 @@ sort_with_function (const void *item1, const void *item2)
   return (result);
 }
 
-/* <sort array-var [sort-fun] [caseless=true]> */
-static void
-pf_sort (PFunArgs)
+DEFUN (pf_sort, arrayvar &optional sort-fun &key caseless=true
+       sortorder=[ascending|descending] number=true,
+"Sort the contents of the array <var arrayvar>.
+
+The elements are sorted in place -- this function has no return value.
+
+If <var caseless=true> is given, then the comparison of the elements of the array is done without regards to case.
+
+If <var sortorder=reverse> is given, then the results are returned in descending order, instead of ascending order.  The default is to order the elements in ascending order.
+
+If <var numeric=true> is given, then the elements of <var arrayvar> are treated as numeric entities, whether they are or not.  The default is to treat the elements as character strings, which can have unexpected results when sorting numeric quantities (\"11\" is less then \"2\" when sorting alphabetically!)
+
+Finally, you may supply a sorting function, whose name is passed as <var sort-fun>.  This function will be called on each element just before comparison, and the results of that function will be used for the comparison instead of the element itself.  This allows you to create a collating sort, or to sort on complex weighting features, or anything else that you can conceive of.
+
+Examples:
+
+Given the array:
+<set-var array[0] = 1
+         array[1] = 2
+         array[3] = 3
+         array[4] = 4
+         array[5] = 20>
+<example>
+<set-var array[0] = 1
+         array[1] = 2
+         array[3] = 3
+         array[4] = 4
+         array[5] = 20>
+</example>
+then,
+
+<complete-example>
+<sort array>
+<foreach x array> <get-var x> </foreach>
+</complete-example>
+while
+<complete-example>
+<sort array numeric=true>
+<foreach x array> <get-var x> </foreach>
+</complete-example>
+
+Sorting strings:
+<complete-example>
+<set-var array[]=\"a\\nb\\nc\\nd\\ne\\nf\\nA\\nB\\nC\\nD\\nE\\nF\">
+<sort array sortorder=descending>
+<foreach x array> <get-var x> </foreach>
+</complete-example>
+
+Without regards to case:
+<complete-example>
+<sort array caseless=true>
+<foreach x array> <get-var x> </foreach>
+</complete-example>
+
+Finally, here is an example which sorts a list
+of words based upon the percentage of vowels
+present in each word, using a sort function
+which calculates that value for each string:
+
+<complete-example>
+<defun vowel-percentage string>
+  <set-var x =
+    <subst-in-string <downcase <get-var string>> \"([^aeiou])\" \"\">>
+  <percent <string-length <get-var x>>
+           <string-length <get-var string>>>
+</defun>
+.blank
+<set-var words[]=
+  <prog
+    Brian
+    Fox
+    sorts
+    elegant
+    strings
+    beautifully>>
+.blank
+<sort words vowel-percentage numeric=true sortorder=descending>
+.blank
+<foreach word words>
+  <get-var word> (<vowel-percentage <get-var word>>)<br>
+</foreach>
+</complete-example>")
 {
   char *sortvar = mhtml_evaluate_string (get_positional_arg (vars, 0));
 
