@@ -43,6 +43,7 @@ static PFunDesc func_table[] =
 };
 
 PACKAGE_INITIALIZER (initialize_cleaner_funcs)
+DOC_SECTION (PAGE-OPERATORS)
 
 #define newline(x) ((x) == '\r' || (x) == '\n')
 static char *
@@ -164,8 +165,16 @@ cleanup_html_whitespace (char *string, int len, int beautify_p)
   return (result);
 }
 	    
-static void
-pf_cleanup_whitespace (PFunArgs)
+DEFUN (pf_cleanup_whitespace, ,
+"Reduces the amount of whitespace in the output page to the minimum
+required.  This function carefully avoids the contents of <example
+code><pre> ... </pre></example> constructs, as well as any material
+appearing in a <example code><textarea ...> ... </textarea></example>
+construct.
+
+Placed as the last instruction on a page, it is an effective way to
+reduce the amount of data sent back over the network to the connecting
+browser.")
 {
   char *varname = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *cleaned = (char *)NULL;
