@@ -40,11 +40,6 @@ static void pf_database_next_key (PFunArgs);
 static void pf_database_unique_key (PFunArgs);
 static void pf_database_query (PFunArgs);
 
-#if defined (NOT_NEEDED_WITH_DATABASE_QUERY)
-static void pf_database_first_match (PFunArgs);
-static void pf_database_next_match (PFunArgs);
-#endif /* NOT_NEEDED_WITH_DATABASE_QUERY */
-
 static PFunDesc func_table[] =
 {
   { "WITH-OPEN-DATABASE",	1, 0, pf_with_open_database },
@@ -56,10 +51,6 @@ static PFunDesc func_table[] =
   { "DATABASE-NEXT-KEY",	0, 0, pf_database_next_key },
   { "DATABASE-UNIQUE-KEY",	0, 0, pf_database_unique_key },
   { "DATABASE-QUERY",		0, 0, pf_database_query },
-#if defined (NOT_NEEDED_WITH_DATABASE_QUERY)
-  { "DATABASE-FIRST-MATCH",	0, 0, pf_database_first_match },
-  { "DATABASE-NEXT-MATCH",	0, 0, pf_database_next_match },
-#endif /* NOT_NEEDED_WITH_DATABASE_QUERY */
   { (char *)NULL,		0, 0, (PFunHandler *)NULL }
 };
 
@@ -163,7 +154,7 @@ lock_database (char *dbname, int open_mode, int *lock, DBFILE *db)
 	{
 	  if (database_had_error_p ())
 	    page_syserr ("%s: %s", dbname, database_strerror ());
-	  /* unlink (lockname); */
+	  unlink (lockname);
 	  UNLOCKFILE (fd);
 	  close (fd);
 	}
@@ -180,22 +171,38 @@ unlock_database (char *dbname, int *lock, DBFILE *db)
 
   if (*lock > -1)
     {
-      /* char *lockname = db_lockname (dbname);
-	 unlink (lockname); */
+      char *lockname = db_lockname (dbname);
+      unlink (lockname); 
       UNLOCKFILE (*lock);
       close (*lock);
     }
 }
 
-/* <with-open-database variable dbname mode=[writer/reader/write-create]>
-   [code using the open database]
-   </with-open-database>
-   Opens the database specified by DBNAME, and stores a referent
-   to it in VARIABLE.  The database is opened in the mode specified
-   by MODE.  If MODE is not specified, the database is opened read-only.
-   If the operation fails, the value of VARIABLE is the empty string.*/
-static void
-pf_with_open_database (PFunArgs)
+DEFMACRO (pf_with_open_database, dbvar dbname &key mode=open-mode,
+"Create an environment in which other database commands can be given.
+First, the database referenced by <var dbname> is locked and opened in
+the mode specified by <var open-mode>, and the resultant database
+handle is bound to the variable named by <var dbvar>.  Then, the <var
+body> code is executed.  Finally, the database is closed, and further
+references to <var dbvar> are meaningless.
+
+Please note that the file name specified by <var dbname> should be a
+full pathname; it is not relative to Web space in any way.
+
+<var open-mode> should evaluate to one of the following:
+
+<ul>
+  <li> <var reader><br>
+The caller wishes only to have read access to the specified database.
+
+<li> <var writer><br>
+The caller wishes to have both read and write access to the specified
+database.
+
+<li> <var write-create><br>
+The caller wishes both read and write access to the specified
+database.  If the database does not exist, it is created.
+</ul>")
 {
   char *varname;
   char *dbname;
@@ -432,9 +439,16 @@ Upon success, this function returns the string <code>\"true\"</code>.
     bprintf_insert (page, start, "%s", result);
 }
 
-/* <database-delete-record DB KEY> */
-static void
-pf_database_delete_record (PFunArgs)
+DEFUN (pf_database_delete_record, dbvar key,
+"Remove the record associated with <var key> from the database
+referenced by <var dbvar>.  This functions returns the string
+<code>true</code> upon success.
+
+<example>
+<with-open-database db /tmp/file.db mode=write>
+  <set-var deleted? = <database-delete-record db <get-var key>>>
+</with-open-database>
+</example>")
 {
   DBFILE db;
   DBOBJ *key;
@@ -459,9 +473,28 @@ pf_database_delete_record (PFunArgs)
     bprintf_insert (page, start, "%s", result);
 }
 
-/* <database-save-record db key var1 var2 ... varN [prefix=string]> */
-static void
-pf_database_save_record (PFunArgs)
+DEFUN (pf_database_save_record, dbvar key &optional var... &key
+       package=package-name,
+"Save the variables <var var1...varn> in the database referenced by
+<var dbvar> in the record specified by the key <var key>.  If <var
+package-name> is supplied, then the names of the variables written to
+the database are written as if they belonged to that package.
+
+Upon success, this function returns the string <code>true</code>.
+
+Also see <funref database-commands database-save-package>, and
+<funref database-commands database-load-record>.
+
+<example>
+<with-open-database db /tmp/file.db mode=write-create>
+  <set-var saved? =
+     <database-save-record db <get-var name> name age size>>
+</with-open-database>
+
+<when <get-var saved?>>
+  The record was stored successfully.
+</when>
+</example>")
 {
   DBFILE db;
   DBOBJ *key;
@@ -524,9 +557,27 @@ pf_database_save_record (PFunArgs)
     bprintf_insert (page, start, "%s", result);
 }
 
-/* <database-save-package db key package [strip=true]> */
-static void
-pf_database_save_package (PFunArgs)
+DEFUN (pf_database_save_package, dbvar key package &key strip=true,
+"Save the variables from the package <var package> in the database
+referenced by <var dbvar> in the record specified by <var key>.
+
+If <var strip=true> is supplied, then the names of the variables
+written to the database have their package prefix removed as they are
+written.
+
+Upon success, this function returns the string <code>true</code>.
+
+<example>
+<set-var foo::x=x foo::array[0]=val0 foo::array[1]=val1>
+<with-open-database db /tmp/file.db mode=write-create>
+  <database-save-package db mykey foo strip=true>
+</with-open-database>
+</example>
+
+produces
+<example>
+true
+</example>")
 {
   DBFILE db;
   DBOBJ *key;
@@ -618,9 +669,12 @@ pf_database_save_package (PFunArgs)
     }
 }
 
-/* <database-first-key dbvar> */
-static void
-pf_database_first_key (PFunArgs)
+DEFUN (pf_database_first_key, dbvar,
+"Return a string representing the "first" key found in the database
+referenced by <var dbvar>.  The key is suitable for input to any of
+the database functions which takes a key as input.  The order in which
+keys are returned appears random, but the return value is always a
+suitable argument to <funref database-commands database-next-key>.")
 {
   DBFILE db = (DBFILE)0;
   DBOBJ *key;
@@ -641,9 +695,24 @@ pf_database_first_key (PFunArgs)
     }
 }
 
-/* <database-next-key dbvar key> */
-static void
-pf_database_next_key (PFunArgs)
+DEFUN (pf_database_next_key, dbvar after-key,
+"Return the \"next\" key in the database.  The key is found by looking
+at <var after-key>, which makes this function suitable for calling in
+a loop.  For example, the following code iterates over an entire
+database.
+
+<example>
+<with-open-database db /file.db mode=read>
+  <set-var key=<database-first-key db>>
+  <while <get-var key>>
+    <package-delete record>
+    <set-var loaded? =
+      <database-load-record db <get-var key> package=record>>
+    Key: <get-var key>, Name: <get-var record::name>
+    <set-var key = <database-next-key db <get-var key>>>
+  </while>
+</with-open-database>
+</example>")
 {
   DBFILE db;
   DBOBJ *inkey = (DBOBJ *)NULL;
@@ -664,9 +733,11 @@ pf_database_next_key (PFunArgs)
   dbobj_free (inkey);
 }
 
-/* <database-unique-key db [suggestion]> --> a unique key. */
-static void
-pf_database_unique_key (PFunArgs)
+DEFUN (pf_database_unique_key, dbvar &optional suggestion,
+"Return a key which is guaranteed to not already exist in the the
+database referenced by <var db>.  If <var suggestion> is supplied,
+that key is tried first, and then subsequent attempts are various
+modifications to the suggestion.")
 {
   DBFILE db;
   DBOBJ *key;
@@ -718,182 +789,6 @@ pf_database_unique_key (PFunArgs)
     }
 }
 
-#if defined (NOT_NEEDED_WITH_DATABASE_QUERY)
-
-static void
-do_database_search (PAGE *page, int start, DBFILE db, DBOBJ *startkey,
-		    char *field_expr, char *search_expr, int fields_only_p)
-{
-  regex_t field_regex, search_regex;
-  regmatch_t offsets[2];
-  int free_field_regex = 0;
-  int free_search_regex = 0;
-
-  /* If the caller specified that they only care about matching the field
-     name, yet the field naem is empty, then return now. */
-  if (fields_only_p && empty_string_p (field_expr))
-    return;
-
-  if (startkey != (DBOBJ *)NULL)
-    {
-      if (field_expr)
-	{
-	  free_field_regex = 1;
-	  regcomp (&field_regex, field_expr, REG_EXTENDED | REG_ICASE);
-	}
-
-      free_search_regex = 1;
-      regcomp (&search_regex, search_expr, REG_EXTENDED | REG_ICASE);
-    }
-
-  while (startkey != (DBOBJ *)NULL)
-    {
-      DBOBJ *content = database_fetch (db, startkey);
-      int matched = 0;
-      char *name, *value;
-
-      /* Unpack the record if there is one. */
-      if ((content != (DBOBJ *)NULL) &&
-	  (content->data != (unsigned char *)NULL))
-	{
-	  WispObject *list = wisp_from_string ((char *)content->data);
-
-	  while (list != NIL)
-	    {
-	      WispObject *pair;
-
-	      pair = CAR (list);
-	      list = CDR (list);
-
-	      name = STRING_VALUE (CAR (pair));
-	      value = STRING_VALUE (CDR (pair));
-
-	      /* If the caller only cares about records which contain
-		 a matching field name, check the name now. */
-	      if (fields_only_p)
-		{
-		  matched = (regexec (&field_regex, name, 1, offsets, 0) == 0);
-		}
-	      else
-		{
-		  if ((empty_string_p (field_expr)) ||
-		      (regexec (&field_regex, name, 1, offsets, 0) == 0))
-		    matched =
-		      (regexec (&search_regex, value, 1, offsets, 0) == 0);
-		}
-
-	      if (matched)
-		break;
-	    }
-	  free (content->data);
-	  free (content);
-	  gc_wisp_objects ();
-	}
-
-      if (matched)
-	{
-	  bprintf_insert (page, start, "%s", startkey->data);
-	  free (startkey->data);
-	  free (startkey);
-	  break;
-	}
-      else
-	{
-	  DBOBJ *nextkey;
-	  nextkey = database_nextkey (db, startkey);
-	  free (startkey->data);
-	  free (startkey);
-	  startkey = nextkey;
-	}
-    }
-
-  if (free_field_regex) regfree (&field_regex);
-  if (free_search_regex) regfree (&search_regex);
-}
-
-/* <database-first-match dbvar field-expr search-expr [fields-only]> */
-static void
-pf_database_first_match (PFunArgs)
-{
-  DBFILE db = (DBFILE)0;
-  char *dbref = mhtml_evaluate_string (get_positional_arg (vars, 0));
-  char *field_expr;
-  char *search_expr;
-  int fields_only_p = var_present_p (vars, "fields-only");
-
-  if (database_environment_level == 0)
-    return;
-
-  field_expr = mhtml_evaluate_string (get_positional_arg (vars, 1));
-  search_expr= mhtml_evaluate_string (get_positional_arg (vars, 2));
-
-  if (search_expr == (char *)NULL)
-    return;
-
-  if (dbref != (char *)NULL)
-    {
-      char *rep = pagefunc_get_variable (dbref);
-
-      if (rep != (char *)NULL)
-	{
-	  long dbval = strtol (rep, (char **)NULL, 16);
-
-	  db = (DBFILE)dbval;
-	}
-    }
-
-  if (db != (DBFILE)0)
-    {
-      DBOBJ *startkey = database_firstkey (db);
-
-      do_database_search
-	(page, start, db, startkey, field_expr, search_expr, fields_only_p);
-    }
-}
-
-/* <database-next-match dbvar startkey field-expr search-expr [fields-only]> */
-static void
-pf_database_next_match (PFunArgs)
-{
-  DBFILE db = (DBFILE)0;
-  char *dbref = mhtml_evaluate_string (get_positional_arg (vars, 0));
-  char *dbkey = mhtml_evaluate_string (get_positional_arg (vars, 1));
-  char *field_expr = mhtml_evaluate_string (get_positional_arg (vars, 2));
-  char *search_expr = mhtml_evaluate_string (get_positional_arg (vars, 3));
-  int fields_only_p = var_present_p (vars, "fields-only");
-
-  if (database_environment_level == 0)
-    return;
-
-  if (empty_string_p (search_expr))
-    return;
-
-  if ((!empty_string_p (dbref)) &&  (!empty_string_p (dbkey)))
-    {
-      char *rep = pagefunc_get_variable (dbref);
-
-      if (rep != (char *)NULL)
-	{
-	  long dbval = strtol (rep, (char **)NULL, 16);
-
-	  db = (DBFILE)dbval;
-	}
-    }
-
-  if (db != (DBFILE)0)
-    {
-      DBOBJ *key = database_setkey (dbkey);
-      DBOBJ *startkey = database_nextkey (db, key);
-      free (key);
-      do_database_search
-	(page, start, db, startkey, field_expr, search_expr, fields_only_p);
-    }
-
-  if (dbkey) free (dbkey);
-  if (dbref) free (dbref);
-}
-#endif /* NOT_NEEDED_WITH_DATABASE_QUERY */
-
 /* Read the record in DB at KEY, and return a package containing the
    fields and values. */
 static Package *
@@ -1086,8 +981,69 @@ dbrec_callout (const void *arg1, const void *arg2)
   return (result);
 }
 
-static void
-pf_database_query (PFunArgs)
+DEFUN (pf_database_query, dbvar expr &key format=fexpr keys=varname
+       sort=field... sortorder=reverse,
+"Select records from the database referenced by <var dbvar>, based on
+<var expr>, perhaps sorting them based on the data in <var FIELD>s,
+and optionally formatting them with the <Meta-HTML> expression in <var
+fexpr>.
+
+Both <var expr> and <var fexpr> are evaluated with the fields of the
+database as the current package, so references to variables which are
+not part of the record structure must be fully qualified (i.e., by
+prefixing them with a package name. See <secref Packages> for more
+information.) If the result of evaluating <var expr> is not the empty
+string, then the record is selected for further processing by either
+<var format>, <var keys>, or to be returned in a  plain text list of
+keys. 
+
+The order of evaluation is as follows:
+
+First, the database is queried, and an internal list of selected
+records is created.  Then, if <var sort=field1,field2...> is present,
+the records are sorted using the values of the specified fields.  The
+order of the sort can be reversed by passed <var sortorder=reverse>. 
+
+Next, if <var format=fexpr> is present, <var fexpr> is executed for
+each record, in an environment where the current package consists of
+bindings for each field's name to each field's value, and the special
+binding of <code>key</code> to the key of that record.  Finally, if
+<var keys=varname> is present, <var varname> is the name of a variable
+into which the matched keys are stored as an array.
+
+The <var sort> parameters indicates which fields to sort the resultant
+keys on; the field names <var field1,field2...> are separated by
+commas.
+
+Examples:
+
+This simple query stores the key of every record in the database whose
+<b>name</b> field contains \"Fox\" into the variable named
+<b>dbkeys</b>:
+
+<example>
+<with-open-database db \"/www/data/employee.db\" mode=read>
+  <database-query db <match <get-var name> \"Fox\"> keys=dbkeys>
+</with-open-database>
+</example>
+
+This more complex query formats an alphabetical list of all the
+members of the database who are employed as carpenters, and are
+between the ages of 20 and 45, by creating links to a page which will
+(presumably) get a more detailed listing of the individual record:
+
+<example>
+<with-open-database db \"employee.db\" mode=read>
+  <database-query db sort=LastName
+    <and <match <get-var job> \"carpent\">
+         <gt age 20>
+         <lt age 45>>
+    format = <prog
+               <a href=\"detail-display.mhtml?<cgi-encode key>\">
+               <get-var LastName>, <get-var FirstName>
+               </a>: <get-var Job>, <get-var Age>>>
+</with-open-database>
+</example>")
 {
   DBFILE db = (DBFILE)0;
   char *dbref = mhtml_evaluate_string (get_positional_arg (vars, 0));
