@@ -279,7 +279,7 @@ association-list which are guaranteed to be present are:
 Example:
 <complete-example>
 <get-file-properties />
-<complete-example>")
+</complete-example>")
 {
   char *filename = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *result = get_file_properties (filename);
@@ -298,9 +298,9 @@ Example:
 DEFUN (pf_file_exists, pathname,
 "Returns \"true\" if <var pathname> is an existing file or directory.
 
-<var pathname> is absolute -- it is looked up in absolute file system
-space, not in Web space.  To find out if a file <code>foo</code> in
-the current directory exists, use:
+<var pathname> is absolute, that is to say that it is looked up in
+absolute file system space, not in Web space.  To find out if a file
+<code>foo</code> in the current directory exists, use:
 <example>
 <file-exists <thisdir>/foo>
 </example>")
@@ -341,10 +341,11 @@ file.")
 {
   char *dirname = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *matching = mhtml_evaluate_string (get_value (vars, "matching"));
+  DIR *dir = (DIR *)NULL;
 
   if (!empty_string_p (dirname))
     {
-      DIR *dir = opendir (dirname);
+      dir = opendir (dirname);
   
       if (dir != (DIR *)NULL)
 	{
@@ -415,6 +416,9 @@ file.")
 	}
     }
 
+  if (dir != (DIR *)NULL)
+    closedir (dir);
+
   if (matching != (char *)NULL) free (matching);
   if (dirname != (char *)NULL) free (dirname);
 }
