@@ -37,32 +37,16 @@
 /*							    */
 /************************************************************/
 
-/* <if test then else> If eval (test) produces non-zero length text, then... */
 static void pf_if (PFunArgs);
-
-/* <ifeq this that then else> If eval (this) = eval (that), then, else */
 static void pf_ifeq (PFunArgs);
-
-/* <when TEST> body </when> */
 static void pf_when (PFunArgs);
-
-/* <var-case [this=that do-this] ...> Do-this where <get-var THIS> == THAT. */
 static void pf_var_case (PFunArgs);
 
-/* <while test>  body-text ... </while> Do BODY-TEXT while TEST
-   produces non-zero length text. */
 #define PAGE_ITERATOR_MAX_COUNT 500 /* No more than this many times. */
 static void pf_while (PFunArgs);
 
-/* <break> Only valid inside of <while> or <foreach>. */
 static void pf_break (PFunArgs);
-
-/* <return &rest args> For functions and while loops. */
 static void pf_return (PFunArgs);
-
-/* <set-var x=hello>
-   <with x=1 z=<get-var foo>> <get-var x> </with> --> 1
-   <get-var x> --> hello */
 static void pf_with (PFunArgs);
 
 /* Return the parser's idea of the current line number. */
@@ -467,7 +451,8 @@ had before encountering the <code>with</code> macro.
   /* Call the (let ((...))) code with arguments almost like ours -- having
      replaced the input variables package with a new one containing the
      evaluated values. */
-  mhtml_let (page, body, newvars, start, end, newstart, debug_level);
+  vars = newvars;
+  mhtml_let (PassPFunArgs);
   symbol_destroy_package (newvars);
 }
 
