@@ -45,6 +45,7 @@ static void pf_database_column_info (PFunArgs);
 static void pf_database_columns_info (PFunArgs);
 static void pf_database_query_info (PFunArgs);
 static void pf_database_num_rows (PFunArgs);
+static void pf_database_affected_rows (PFunArgs);
 static void pf_database_set_pos (PFunArgs);
 static void pf_database_save_record (PFunArgs);
 static void pf_database_load_record (PFunArgs);
@@ -81,6 +82,14 @@ static void pf_sql_transact (PFunArgs);
 #define GSQL_UINT           104
 #define GSQL_ANY            105
 
+/* MYSQL extensions. (tamsky@as.ucsb.edu: Sat Oct 11 11:55:38 1997) */
+#define GSQL_TINY_BLOB      249
+#define GSQL_MEDIUM_BLOB    250
+#define GSQL_LONG_BLOB      251
+#define GSQL_BLOB           252
+#define GSQL_VAR_STRING     253
+#define GSQL_STRING         254
+
 /* Reporting of db errors to Meta-HTML environment  */
 #define GSQL_DEFAULT_ERRMSG NULL
 
@@ -184,8 +193,12 @@ istack_free (IStack *stack, FREEFUN *free_fun)
 #  include "msqlfuncs.c"
 #endif
 
-#if defined (COMPILING_IODBCFUNCS)
-#  include "iodbcfuncs.c"
+#if defined (COMPILING_MYSQLFUNCS)
+#  include "mysqlfuncs.c"
+#endif
+
+#if defined (COMPILING_ODBCFUNCS)
+#  include "odbcfuncs.c"
 #endif
 
 /* Parameters:
@@ -291,8 +304,7 @@ free_cursor (DBCursor *cursor)
 static DBCursor *
 lookup_cursor_ref (char *cursor_ref)
 {
-
-  Database * db;
+  Database *db;
   int cursor_token = -1;		/* The combined db::cursor ids */
   int cursor_id;
   int db_id;
@@ -355,11 +367,17 @@ get_cursor_ref (Package *vars)
 #define CHARTYPE(ftype) ((! (gsql_field_type (ftype) == GSQL_NUMERIC)) \
 			 && (! (gsql_field_type (ftype) == GSQL_DECIMAL)) \
 			 && (! (gsql_field_type (ftype) == GSQL_INTEGER)) \
+			 && (! (gsql_field_type (ftype) == GSQL_UINT)) \
 			 && (! (gsql_field_type (ftype) == GSQL_SMALLINT)) \
 			 && (! (gsql_field_type (ftype) == GSQL_FLOAT)) \
 			 && (! (gsql_field_type (ftype) == GSQL_REAL)) \
 			 && (! (gsql_field_type (ftype) == GSQL_DOUBLE)))
 
+#define BLOBTYPE(field) ((gsql_field_type (field) == GSQL_TINY_BLOB) || \
+			 (gsql_field_type (field) == GSQL_MEDIUM_BLOB) || \
+			 (gsql_field_type (field) == GSQL_LONG_BLOB) || \
+			 (gsql_field_type (field) == GSQL_BLOB))
+
 /* We pass an string which contains a numeric representation of the
    SQL escape code character, such as "39" */
 static void
@@ -387,12 +405,12 @@ gsql_database_sql_escape_character (Database *db)
 static void
 gsql_database_set_truncate_columns (Database *db, char *value)
 {
-  int truncate_p;
+  int truncate_p = DEFAULT_SQL_TRUNCATE_COLUMNS;
 
-  if (!empty_string_p (value))
-    truncate_p = 1;
+  if (empty_string_p (value))
+    truncate_p = 0;
   else
-    truncate_p = DEFAULT_SQL_TRUNCATE_COLUMNS;
+    truncate_p = 1;
 
   db->sql_truncate_columns = truncate_p;
 }
@@ -414,7 +432,7 @@ gsql_database_set_prefix_tablenames (Database *db, char *value)
   if (!empty_string_p (value))
     prefix_p = 1;
   else
-    prefix_p = DEFAULT_SQL_PREFIX_TABLENAMES;
+    prefix_p = 0;
 
   db->sql_prefix_tablenames = prefix_p;
 }
@@ -533,24 +551,6 @@ get_dbref (Package *vars)
   return (get_dbref_internal (vars, &dbindex));
 }
 
-/* <with-open-database variable DSN=dsn-string [DATABASE=dbname HOST=hostname]
-                                [NOLOCK=TRUE]>
-   [code using the open database]
-   </with-open-database>
-
-   Opens the database specified by DSN (database service name) string, and
-   stores a referent to it in VARIABLE.  If the db-connect fails,
-   value of VARIABLE is the empty string.
-
-   The DSN should contain a minimum of "HOST=hostname;DATABASE=dbname"
-   The older HOST=host DATABASE=dbname is still supported, but the
-   DSN string is preferred.
-
-   If NOLOCK supplied, then no lock file on the server is created.
-   This is useful if the SQL database supports already supports
-   transactions, and you don't want to block other processes from
-   accessing the database for the duration of the with-open-database
-   form. */
 static void
 pf_with_open_database (PFunArgs)
 {
@@ -780,7 +780,7 @@ pf_database_exec_query (PFunArgs)
 		  cursor_token =
 		    ((dbindex & 0xff) << 8) | (cursor->index & 0xff);
 
-		  sprintf (cursor_string, "%x", cursor_token);
+		  sprintf (cursor_string, "0X%04X", cursor_token);
 
 		  if (!empty_string_p (cursorvar))
 		    {
@@ -815,6 +815,18 @@ pf_database_num_rows (PFunArgs)
     }
 }
 
+static void
+pf_database_affected_rows (PFunArgs)
+{
+  DBCursor *cursor = get_cursor_ref (vars);
+
+  if (cursor != (DBCursor *)NULL)
+    {
+      int nrows = gsql_affected_rows (cursor);
+      bprintf_insert (page, start, "%d", nrows);
+    }
+}
+
 /* Set the current row number position in the result set from the last query.
    This operates on a CURSOR object. */
 static void
@@ -938,33 +950,33 @@ free_array (char **array)
     }
 }
 
-/* Split STRING into an array of words.  The boundaries are whitespace
+/* Split TEXT into an array of words.  The boundaries are whitespace
    and commas. Returns the number of words found in NCOLS.*/
 static char **
-split_colnames (char *string, int *ncols)
+split_colnames (char *text, int *ncols)
 {
   char **result = (char **)NULL;
   int result_slots = 0;
   int result_index = 0;
 
-  if (string != (char *)NULL)
+  if (text != (char *)NULL)
     {
       register int i = 0;
 
       while (1)
 	{
 	  /* Skip leading separator characters. */
-	  while ((whitespace (string[i])) || (string[i] == ',')) i++;
+	  while ((whitespace (text[i])) || (text[i] == ',')) i++;
 
-	  if (string[i] == '\0')
+	  if (text[i] == '\0')
 	    break;
 	  else
 	    {
 	      int start = i;
 
-	      while ((string[i] != '\0') &&
-		     (!whitespace (string[i])) &&
-		     (string[i] != ','))
+	      while ((text[i] != '\0') &&
+		     (!whitespace (text[i])) &&
+		     (text[i] != ','))
 		i++;
 
 	      if (start < i)
@@ -974,7 +986,7 @@ split_colnames (char *string, int *ncols)
 		      (result, (result_slots += 10) * sizeof (char *));
 
 		  result[result_index] = (char *)xmalloc (1 + (i - start));
-		  strncpy (result[result_index], string + start, i - start);
+		  strncpy (result[result_index], text + start, i - start);
 		  result[result_index][i - start] = '\0';
 		  result_index++;
 		  result[result_index] = (char *)NULL;
@@ -1099,29 +1111,87 @@ pf_database_next_record (PFunArgs)
 	      char *fieldname= (char *) NULL;
 	      int found_next = 0;
 	      int ncolnames = 0;
-	      Package *package = symbol_get_package (pkgname);
+	      Package *package;
 
-	      {
-		char *t = mhtml_evaluate_string (get_value (vars, "COLNAMES"));
-
-		if (!empty_string_p (t))
-		  fieldnames = split_colnames (t, &ncolnames);
-
-		xfree (t);
-	      }
-
-	      /* If the user does not want variables to have their
-		 appended values persist across calls to next-record,
-		 then we loop here to null out all the symbols which
-		 are going to be set in this result row. */
-	      if (empty_string_p (appendvars))
+	      if (empty_string_p (pkgname))
 		{
-		  for (i = 0; i < limit; i++)		  
+		  /* We can't do anything with the variables, so don't
+		     bother.  We've already fetched the row, so the
+		     cursor object has been advanced. */
+		  found_next = 1;
+		}
+	      else
+		{
+		  char *t = mhtml_evaluate_string
+		    (get_value (vars, "COLNAMES"));
+
+		  if (!empty_string_p (t))
+		    fieldnames = split_colnames (t, &ncolnames);
+
+		  xfree (t);
+
+		  package = symbol_get_package (pkgname);
+
+		  /* If the user does not want variables to have their
+		     appended values persist across calls to next-record,
+		     then we loop here to null out all the symbols which
+		     are going to be set in this result row. */
+		  if (empty_string_p (appendvars))
+		    {
+		      for (i = 0; i < limit; i++)
+			{
+			  gsql_field *field = gsql_fetch_field
+			    (query_result, i);
+			  char buf[256];
+			  Symbol *sym;
+
+			  /* If the user supplied column names, use them. */
+			  if (fieldnames != (char **)NULL)
+			    {
+			      if (i < ncolnames)
+				fieldname = fieldnames[i];
+			      else
+				fieldname = (char *) NULL;
+			    }
+			  else
+			    fieldname = gsql_field_name (field);
+
+			  if (empty_string_p (fieldname))
+			    break;
+			  else
+			    {
+			      if (gsql_field_table (field) &&
+				  (gsql_database_sql_prefix_tablenames
+				   (cursor->db)
+				   || !(empty_string_p (prefixtablenames))))
+				{
+				  strcpy (buf, gsql_field_table (field));
+				  strcat (buf, ".");
+				  strcat (buf, fieldname);
+				}
+			      else
+				{
+				  strcpy (buf, fieldname);
+				}
+
+			      sym = symbol_intern_in_package (package, buf);
+			      symbol_reset (sym);
+			    }
+			}
+		    }
+
+		  /* Loop over fields of the row, binding variables.
+		     All data is already coerced to strings, so we
+		     do not do any type conversion. */
+		  for (i = 0; i < limit; i++)
 		    {
 		      gsql_field *field = gsql_fetch_field (query_result, i);
 		      char buf[256];
 		      Symbol *sym;
-		      
+
+		      if (!empty_string_p (fieldname))
+			prev_fieldname = fieldname;
+
 		      /* If the user supplied column names, use them. */
 		      if (fieldnames != (char **)NULL)
 			{
@@ -1132,11 +1202,18 @@ pf_database_next_record (PFunArgs)
 			}
 		      else
 			fieldname = gsql_field_name (field);
-
-		      if (empty_string_p (fieldname))
+		      
+		      if (empty_string_p (fieldname) &&
+			  empty_string_p (prev_fieldname))
 			break;
 		      else
 			{
+			  char *value;
+			  found_next++;
+			  
+			  if (empty_string_p (fieldname))
+			    fieldname = prev_fieldname;
+
 			  if (gsql_field_table (field) &&
 			      (gsql_database_sql_prefix_tablenames (cursor->db)
 			       || !(empty_string_p (prefixtablenames))))
@@ -1151,66 +1228,14 @@ pf_database_next_record (PFunArgs)
 			    }
 
 			  sym = symbol_intern_in_package (package, buf);
-			  symbol_reset (sym);
+			  value = gsql_get_column (query_result, i);
+			  symbol_add_value (sym, value ? value : "");
+
+			  xfree (value);
 			}
 		    }
 		}
 
-	      /* Loop over fields of the row, binding variables.
-		 All data is already coerced to strings, so we
-		 do not do any type conversion. */
-	      for (i = 0; i < limit; i++)
-		{
-		  gsql_field *field = gsql_fetch_field (query_result, i);
-		  char buf[256];
-		  Symbol *sym;
-
-		  if (!empty_string_p (fieldname))
-		    prev_fieldname = fieldname;
-
-		  /* If the user supplied column names, use them. */
-		  if (fieldnames != (char **)NULL)
-		    {
-		      if (i < ncolnames)
-			fieldname = fieldnames[i];
-		      else
-			fieldname = (char *) NULL;
-		    }
-		  else
-		    fieldname = gsql_field_name (field);
-
-		  if (empty_string_p (fieldname) &&
-		      empty_string_p (prev_fieldname))
-		    break;
-		  else
-		    {
-		      char *value;
-		      found_next++;
-
-		      if (empty_string_p (fieldname))
-			  fieldname = prev_fieldname;
-
-		      if (gsql_field_table (field)
-			  && (gsql_database_sql_prefix_tablenames (cursor->db)
-			      || !(empty_string_p (prefixtablenames))))
-			{
-			  strcpy (buf, gsql_field_table (field));
-			  strcat (buf, ".");
-			  strcat (buf, fieldname);
-			}
-		      else
-			{
-			  strcpy (buf, fieldname);
-			}
-
-		      sym = symbol_intern_in_package (package, buf);
-		      value = gsql_get_column (query_result, i);
-		      symbol_add_value (sym, value ? value : "");
-
-		      xfree (value);
-		    }
-		}
-
 	      if (found_next)
 		{
 		  bprintf_insert (page, start, "true");
@@ -1545,7 +1570,7 @@ pf_database_query (PFunArgs)
       gsql_result *query_result = (gsql_result *)NULL;
 
       if (query == (char *)NULL)
-	query = mhtml_evaluate_string (get_positional_arg (vars, 1));
+	query = mhtml_evaluate_string (get_positional_arg (vars, 2));
 
       /* Execute the query. */
       if (!empty_string_p (query))
@@ -1570,6 +1595,10 @@ pf_database_query (PFunArgs)
 	  int ncolnames = 0;
 	  int window_limit = window_length;
 
+	  /* Optimize most common case. */
+	  if ((expr != (char *)NULL) && (strcasecmp (expr, "true") == 0))
+	    expr = (char *)NULL;
+
 	  if (!empty_string_p (colnames))
 	    fieldnames = split_colnames (colnames, &ncolnames);
 
@@ -1825,11 +1854,11 @@ bprintf_gsql_escape (BPRINTF_BUFFER *out, char *src, gsql_field *field,
   escape_string[0] = escape_code;
   escape_string[1] = (char) 0;
 
-  /*  We only truncate character type */
+  /*  We only truncate character types. */
   if (field != (gsql_field *)NULL)
     {
       fieldtype = gsql_field_type (field);
-      if (CHARTYPE(field))
+      if ((CHARTYPE (field)) && (!BLOBTYPE (field)))
 	maxlength = gsql_field_length (field);
     }
 
@@ -1837,7 +1866,7 @@ bprintf_gsql_escape (BPRINTF_BUFFER *out, char *src, gsql_field *field,
     {
       /* If the src string is empty,
 	 SQL fields with numeric types get NULL values. */
-      if (!(CHARTYPE(field)))
+      if (!(CHARTYPE (field)))
 	bprintf (out, "NULL");
       return;
     }
@@ -1920,7 +1949,10 @@ lookup_fieldname (char *name, gsql_result *result)
 
 #define QUOTE_KEYVAL_IF_NEEDED \
 	if ((gsql_field_type (keyfield) == GSQL_CHAR) || \
-	    (gsql_field_type (keyfield) == GSQL_VARCHAR)) bprintf (query, "'")
+	    (gsql_field_type (keyfield) == GSQL_VARCHAR) || \
+	    (gsql_field_type (keyfield) == GSQL_VAR_STRING) || \
+	    (gsql_field_type (keyfield) == GSQL_STRING)) \
+	  bprintf (query, "'")
 
 static void
 pf_database_save_record (PFunArgs)
@@ -2089,8 +2121,7 @@ pf_database_save_record (PFunArgs)
 	  /* This is used to tell if we need to print a comma */
 	  items_printed = 0;
 
-	  while ((name = get_positional_arg (vars, position))
-		 != (char *)NULL)
+	  while ((name = get_positional_arg (vars, position)) != (char *)NULL)
 	    {
 	      char *value = pagefunc_get_variable (name);
 	      Symbol *sym;
@@ -2665,25 +2696,34 @@ pretty_field_type (int fieldtype)
 {
   switch (fieldtype)
     {
-    case  GSQL_CHAR:     return "GSQL_CHAR";
-    case  GSQL_NUMERIC:  return "GSQL_NUMERIC";
-    case  GSQL_DECIMAL:  return "GSQL_DECIMAL";
-    case  GSQL_INTEGER:  return "GSQL_INTEGER";
-    case  GSQL_SMALLINT: return "GSQL_SMALLINT";
-    case  GSQL_FLOAT:    return "GSQL_FLOAT";
-    case  GSQL_REAL:     return "GSQL_REAL";
-    case  GSQL_DOUBLE:   return "GSQL_DOUBLE";
-    case  GSQL_VARCHAR:  return "GSQL_VARCHAR";
+    case  GSQL_CHAR:       return ("GSQL_CHAR");
+    case  GSQL_NUMERIC:    return ("GSQL_NUMERIC");
+    case  GSQL_DECIMAL:    return ("GSQL_DECIMAL");
+    case  GSQL_INTEGER:    return ("GSQL_INTEGER");
+    case  GSQL_SMALLINT:   return ("GSQL_SMALLINT");
+    case  GSQL_FLOAT:      return ("GSQL_FLOAT");
+    case  GSQL_REAL:       return ("GSQL_REAL");
+    case  GSQL_DOUBLE:     return ("GSQL_DOUBLE");
+    case  GSQL_VARCHAR:    return ("GSQL_VARCHAR");
 
-    case GSQL_IDENT:     return "GSQL_IDENT";
-    case GSQL_NULL:      return "GSQL_NULL";
-    case GSQL_IDX:       return "GSQL_IDX";
-    case GSQL_SYSVAR:    return "GSQL_SYSVAR";
-    case GSQL_UINT:      return "GSQL_UINT";
-    case GSQL_ANY:       return "GSQL_ANY";
+      /* MYSQL Extensions. */
+    case GSQL_TINY_BLOB:   return ("GSQL_TINY_BLOB");
+    case GSQL_MEDIUM_BLOB: return ("GSQL_MEDIUM_BLOB");
+    case GSQL_LONG_BLOB:   return ("GSQL_LONG_BLOB");
+    case GSQL_BLOB:	   return ("GSQL_BLOB");
+    case GSQL_VAR_STRING:  return ("GSQL_VAR_STRING");
+    case GSQL_STRING:	   return ("GSQL_STRING");
+
+      /* Non-ANSI Extensions. */
+    case GSQL_IDENT:       return ("GSQL_IDENT");
+    case GSQL_NULL:        return ("GSQL_NULL");
+    case GSQL_IDX:         return ("GSQL_IDX");
+    case GSQL_SYSVAR:      return ("GSQL_SYSVAR");
+    case GSQL_UINT:        return ("GSQL_UINT");
+    case GSQL_ANY:         return ("GSQL_ANY");
       
     default:
-      return "GSQL_CHAR";
+      return ("GSQL_CHAR");
     }
 }
 
@@ -2782,7 +2822,6 @@ pf_database_columns_info (PFunArgs)
 		  colnames = (char **)xmalloc ((cols + 1) * sizeof (char *));
 
 		  /* Reset the field descriptor cursor to start of record */
-
 		  for (i = 0; i < cols; i++)
 		    {
 		      gsql_field *field = gsql_fetch_field (result, i);
@@ -3003,7 +3042,7 @@ pf_database_tables_info (PFunArgs)
 	  /* This returns a result set, with one row of info 
 	     for each table in the database. */
 	  gsql_result *result =
-	    gsql_db_list_tables (db, qualifier, owner,name, type);
+	    gsql_db_list_tables (db, qualifier, owner, name, type);
 
 	  if (result != (gsql_result *)NULL)
 	    {
