@@ -46,6 +46,7 @@ static PFunDesc func_table[] =
   { "MSQL::DATABASE-LOAD-RECORD",	0, 0, pf_database_load_record },
   { "MSQL::DATABASE-SAVE-PACKAGE",	0, 0, pf_database_save_package },
   { "MSQL::NUMBER-OF-ROWS",   	        0, 0, pf_database_num_rows},
+  { "MSQL::AFFECTED-ROWS",   	        0, 0, pf_database_affected_rows},
   { "MSQL::SET-ROW-POSITION",  	        0, 0, pf_database_set_pos},
   { "MSQL::DATABASE-QUERY",		0, 0, pf_database_query },
   { "MSQL::HOST-DATABASES",	        0, 0, pf_host_databases },
@@ -130,7 +131,6 @@ typedef struct
   int sock;
   char *dbname;
   char *hostname;
-
 } Database;
 
 typedef struct
@@ -169,8 +169,15 @@ static void free_database_cursors (Database *db);
 static void
 free_database_resources (Database *db)
 {
-  /* free cursors, namestrings, hdbc resources, etc */
-  free_database_cursors (db);
+  /* Free cursors, namestrings, hdbc resources, etc */
+  if (db != (Database *)NULL)
+    {
+      free_database_cursors (db);
+      xfree (db->dbname);
+      xfree (db->hostname);
+      db->dbname = (char *)NULL;
+      db->hostname = (char *)NULL;
+    }
 }
 
 /* Index where the next error message should go. */
@@ -196,7 +203,7 @@ gsql_save_error_message (Database *db, char *msg)
     msg = (char *)&msqlErrMsg;
 
   pagefunc_set_variable (error_variable, msg);
-  page_debug ("MSQL: %s", msg);
+  page_debug ("mSQL: %s", msg);
 }
 
 static int
@@ -205,7 +212,16 @@ gsql_number_of_rows (gsql_result *result)
   return (msqlNumRows (result->msql_result));
 }
 
-void
+static int
+gsql_affected_rows (DBCursor *cursor)
+{
+  if ((cursor != (DBCursor *)NULL) &&
+      (cursor->result != (gsql_result *)NULL))
+    return (gsql_number_of_rows (cursor->result));
+  return (0);
+}
+
+static void
 gsql_data_seek (gsql_result *result, int position)
 {
   if (result->msql_result != (m_result *)NULL)
@@ -237,10 +253,13 @@ static int
 gsql_query (Database *db, char *query, int save_errors_p)
 {
   int status =  msqlQuery (db->sock, query);
+
+  pagefunc_set_variable ("msql::recent-query", query);
+
   if (status == -1)
     {
       if (save_errors_p)
-	gsql_save_error_message (db, "msqlQuery");
+	gsql_save_error_message (db, GSQL_DEFAULT_ERRMSG);
 
       return (GSQL_ERROR);
     }
@@ -497,18 +516,43 @@ gsql_connect (char *dsn, Database *db)
   db->dbname = dbname;
   db->hostname = dbhost;
 
-  if (dbhost != (char *) NULL)
+  if ((dbhost != (char *) NULL) && (dbname != (char *)NULL))
     {
       sock = msqlConnect (dbhost);
 
       db->sock = sock;
+
       if (sock != -1)
 	{
 	  if (msqlSelectDB (sock, dbname) == -1)
-	    gsql_save_error_message (db, GSQL_DEFAULT_ERRMSG);
+	    {
+	      BPRINTF_BUFFER *e = bprintf_create_buffer ();
+	      bprintf (e, "Unable to connect to database: `%s' on `%s'",
+		       dbname, dbhost);
+	      gsql_save_error_message (db, e->buffer);
+	      bprintf_free_buffer (e);
+	    }
 	  else
 	    db->connected = 1;
 	}
+      else
+	{
+	  BPRINTF_BUFFER *e = bprintf_create_buffer ();
+	  bprintf (e, "Unable to connect to server! ");
+	  bprintf (e, "(HOST: `%s'; DATABASE: `%s')", dbhost, dbname);
+	  gsql_save_error_message (db, e->buffer);
+	  bprintf_free_buffer (e);
+	}
+    }
+  else
+    {
+      BPRINTF_BUFFER *e = bprintf_create_buffer ();
+      bprintf (e, "Unable to connect to server! ");
+      bprintf (e, "(HOST: `%s'; DATABASE: `%s')", 
+	       dbhost ? dbhost : "[not supplied]",
+	       dbname ? dbname : "[not supplied]");
+      gsql_save_error_message (db, e->buffer);
+      bprintf_free_buffer (e);
     }
 }
 
@@ -582,7 +626,8 @@ pf_host_databases (PFunArgs)
     }
   else
     {
-      pagefunc_set_variable ("msql::msql-error-message[]", msqlErrMsg);
+      gsql_save_error_message
+	((Database *)NULL, "HOST-DATABASES: mSQL Connect Failed");
     }
 
   xfree (host);
