@@ -77,12 +77,6 @@ static void pf_page_insert (PFunArgs);
 static void pf_page_search (PFunArgs);
 #endif /* NOT_AT_THIS_TIME */
 
-#if defined (DEPRECATED)
-/* <html-quote "<this is an mhtml command>"> -> &ltthis is ...and&gt */
-static void pf_html_quote (PFunArgs);
-
-#endif /* DEPRECATED */
-
 /************************************************************/
 /*							    */
 /*			HTML "Helper" Functions		    */
@@ -101,11 +95,6 @@ static PFunDesc func_table[] =
   { "COMMENT",		1, 0, pf_comment },
   { "VERBATIM",		1, 0, pf_verbatim },
 
-#if defined (DEPRECATED)
-  /* HTML "Helper" Functions. */
-  { "INPUT-ITEM",	0, 0, pf_input_item },
-#endif /* DEPRECATED */
-
   /* HTML Character manipulation. */
   { "SMALL-CAPS",	1, 0, pf_small_caps },
 
@@ -296,33 +285,16 @@ value is given, then that value is placed into the page instead.")
 
 	  if (file_contents)
 	    {
-#if defined (NOT_BINARY_COMPATIBLE)
-	      bprintf_insert (page, start, "%s", file_contents->buffer);
-#else
+
 	      if (found && !verbatim_p)
 		{
-		  bprintf_insert (file_contents, 0,
-				  "<*parser*::push-file %s>",
+		  bprintf_insert (file_contents, 0, "<*parser*::push-file %s>",
 				  page_last_read_filename);
 		  bprintf (file_contents, "<*parser*::pop-file>");
 		}
 
-	      /* Manually insert the file instead of letting bprintf
-		 do it for us.  This is because the file could contain
-		 binary data, and then file->bindex wouldn't necessarily
-		 reflect the length of what was inserted. */
-	      if ((file_contents->bindex + page->bindex) >= page->bsize)
-		page->buffer = (char *)xrealloc
-		(page->buffer, (page->bsize += (file_contents->bindex + 100)));
-
-	      memmove (page->buffer + start + file_contents->bindex,
-		       page->buffer + start,
-		       (page->bindex + 1) - start);
-
-	      memcpy (page->buffer + start, file_contents->buffer,
-		      file_contents->bindex);
-	      page->bindex += file_contents->bindex;
-#endif /* BINARY_COMPATIBLE */
+	      bprintf_insert_binary
+		(page, start, file_contents->buffer, file_contents->bindex);
 
 	      if (verbatim_p)
 		*newstart += file_contents->bindex;
@@ -582,6 +554,7 @@ occurrences of <code>%name%</code> with a the value of the variable
     }
 
   *top_page->search_start_modified = top_page->start;
+  if (top_page->page == page) *newstart = top_page->start;
 }
 
 #if defined (NOT_AT_THIS_TIME)
@@ -757,13 +730,23 @@ the following redirections are seen:
    Immediately sends this stuff down the line, but doesn't affect
    processesing or the current page. */
 DEFMACRO (pf_server_push, &key type=mime-type,
-"This command will be made obsolete.
+"Writes the appropriate headers and HTTP information to the connected
+browser, along with the data that you specify.
 
-Its functionality is available other, cleaner ways, specifically
-through the use of <funref stream-operators with-open-stream> and the
-variable <varref *standard-output*>.
+The functionality provided by this convenience function is available
+other ways, specifically through the use of <funref stream-operators
+with-open-stream> and the variable <varref *standard-output*>.
 
-Just write it as a <funref macro-commands defmacro>.")
+<example>
+<server-push>
+  <html>
+  <head><title>Just a Moment, please</title></head>
+  <body>
+  <h3>Please wait a moment, we are searching the entire Web...</h3>
+  </body>
+  </html>
+</server-push>
+</example>")
 {
   PAGE *text = (PAGE *)NULL;
   static int called_yet = 0;
@@ -818,7 +801,6 @@ Just write it as a <funref macro-commands defmacro>.")
     }
 }
 
-
 DOC_SECTION (HTML-HELPERS)
 DEFUN (pf_verbatim, &key quote,
 "Insert <var body> verbatim, avoiding doing any processing on
@@ -852,13 +834,16 @@ Contrast this with <funref page-operators comment>.")
 
 DOC_SECTION (LANGUAGE-OPERATORS)
 DEFUN (pf_time, ,
-"Returns the current local time, as measured in seconds since the epoch (12 midnight, December 31, 1969).
+"Returns the current local time, as measured in seconds since the
+epoch (12 midnight, December 31, 1969).
 
-This is often useful as input to the <funref language-operators date> operator.  Here is how one might find out the printable date for the time 10 hours from now.
+This is often useful as input to the <funref language-operators date>
+operator.  Here is how one might find out the printable date for the
+time 10 hours from now.
 
 <example>
    <date> --> Wed Jul  3 17:14:53 1996
-   <date <add <time> <mul 60 <mul 10 60>>>>
+   <date <add <time> <mul 60 60 10>>>
           --> Thu Jul  4 03:14:57 1996
 </example>")
 {
@@ -867,7 +852,7 @@ This is often useful as input to the <funref language-operators date> operator.
   bprintf_insert (page, start, "%ld", ticks);
 }
 
-DEFUN (pf_date, &optional seconds-since-epoch,
+DEFUN (pf_date, &optional seconds-since-epoch &key gmt=true,
 "Returns the local date as a string of 24 characters.
 
 The output looks like <code><date></code>.
@@ -876,6 +861,17 @@ Given the optional argument <var epoch-seconds>, this number is
 treated as the number of seconds since 12 midnight, December 31st,
 1969, and is converted into the current date.
 
+When the keyword argument <var gmt=true> is supplied, returns the date at
+the meridian (Greenwich Mean Time) in the same format as used by HTTP headers,
+such as the <b>Expires</b> header.
+
+Examples:
+<example>
+<date>                  --> Thu Nov 13 10:52:07 1997
+<date gmt=true>         --> Thu, 13 Nov 1997 18:52:07 GMT
+<date <add <time> 60>>  --> Thu Nov 13 10:53:07 1997
+</example>
+
 Also see the <funref language-operators time> function.")
 {
   char *tstring = mhtml_evaluate_string (get_positional_arg (vars, 0));
@@ -891,7 +887,7 @@ Also see the <funref language-operators time> function.")
 
   bprintf_insert (page, start, "%s", time_string);
   *newstart += strlen (time_string);
-  if (tstring) free (tstring);
+  xfree (tstring);
 }
 
 DEFUN (pf_pid, ,
