@@ -279,7 +279,8 @@ page_subst_in_page_pivot (PAGE *page, char *this, char *with_that, int *pivot)
 
       while ((pos = strchr (temp, '\\')) != (char *)NULL)
 	{
-	  if (strchr ("0123456789", pos[1]) != (char *)NULL)
+	  if ((pos[1] != '\0') &&
+	      (strchr ("0123456789", pos[1]) != (char *)NULL))
 	    {
 	      if (ro_index + 2 > ro_slots)
 		replacement_offsets = (int *)xrealloc
@@ -952,7 +953,16 @@ page_find_tag_start (PAGE *page, char *tag, int start)
 
   for (i = start; i < limit; i++)
     {
-      if ((page->buffer[i] == '<') || (page->buffer[i] == LEFT_BRACKET))
+      c = page->buffer[i];
+      if ((c == ';') && (i + 3 < limit) &&
+	  ((page->buffer[i + 1] == ';') && (page->buffer[i + 2] == ';')))
+	{
+	  i += 3;
+	  while ((i < limit) && (page->buffer[i] != '\n')) i++;
+	  continue;
+	}
+
+      if ((c == '<') || (c == LEFT_BRACKET))
 	{
 	  c = page->buffer[i + 1 + len];
 
