@@ -49,6 +49,10 @@ Package *mhtml_function_package = (Package *)NULL;
 /* Macro writing and processing. */
 Package *mhtml_user_keywords = (Package *)NULL;
 
+/* Variable specifically for the mdb debugger.  This allows the setting
+   of a callback function when an mdb breakpoint is hit. */
+DEBUGGER_CALLBACK_FUNCTION *mhtml_debug_callback_function = NULL;
+
 /* The "DEFAULT" package. */
 Package *PageVars = (Package *)NULL;
 
@@ -65,7 +69,7 @@ page_return_this_page (PAGE *page)
   longjmp (page_jmp_buffer, 1);
 }
 
-/* Sequentialy process PAGE. */
+/* Sequentially process PAGE. */
 static PAGE *ThePage = (PAGE *)NULL;
 static int TheOffset = 0;
 
@@ -634,7 +638,7 @@ pagefunc_get_descriptor (char *tag)
 
   sym = symbol_lookup_in_package (mhtml_function_package, tag);
 
-  if (sym != (Symbol *)NULL)
+  if ((sym != (Symbol *)NULL) && (sym->type == symtype_FUNCTION))
     desc = (PFunDesc *)(sym->values);
 
   return (desc);
@@ -799,7 +803,7 @@ parser_trace_indentation (void)
 
       blanks = (char *)xrealloc
 	(blanks, (blanks_size += (indent_amount + 50)));
-      for (i = 0; i < blanks_size; i++)
+      for (i = 0; i < blanks_size - 1; i++)
 	blanks[i] = ' ';
       blanks[i] = '\0';
     }
@@ -1013,8 +1017,15 @@ page_process_page_internal (PAGE *page)
 		  memmove (open_body, open_body + 1 + fname_len,
 			   (open_body_len - (1 + fname_len)));
 
-		  vars = pagefunc_snarf_vars
-		    (open_body, uf ? (uf->flags & user_ACCEPT_KEYWORDS) : 1);
+		  {
+		    int allow_keywords = 1;
+
+		    if (uf)
+		      allow_keywords = (uf->flags & user_ACCEPT_KEYWORDS);
+		    /* else
+		      allow_keywords = (desc->flags & user_ACCEPT_KEYWORDS); */
+		    vars = pagefunc_snarf_vars (open_body, allow_keywords);
+		  }
 
 		  if (!complexp)
 		    {
@@ -1114,6 +1125,8 @@ page_process_page_internal (PAGE *page)
 
 		      parser_trace_indentation_counter++;
 		      {
+			int debug_continue = 1;
+
 #if defined (METAHTML_PROFILER)
 			struct timeval start_time = { 0, 0 };
 			struct timeval end_time = { 0, 0 };
@@ -1122,14 +1135,27 @@ page_process_page_internal (PAGE *page)
 			  gettimeofday (&start_time, (struct timezone *)NULL);
 #endif
 			page_push_page (page, start, &search_start_modified);
-			if (uf)
-			  mhtml_execute_function
-			    (uf, page, body, vars, start, end, &search_start,
-			     desc->debug_level, open_body);
-			else
-			  (*desc->fun)
-			    (page, body, vars, start, end, &search_start,
-			     desc->debug_level);
+
+			if ((desc->debug_level < 0) &&
+			    (mhtml_debug_callback_function !=
+			     (DEBUGGER_CALLBACK_FUNCTION *)NULL))
+			  {
+			    debug_continue =
+			      (*mhtml_debug_callback_function)
+			      (page, body, vars, start, end, &search_start,
+			       0, desc, uf, open_body);
+			  }
+			if (debug_continue)
+			  {
+			    if (uf)
+			      mhtml_execute_function
+				(uf, page, body, vars, start, end,
+				 &search_start, desc->debug_level, open_body);
+			    else
+			      (*desc->fun)
+				(page, body, vars, start, end, &search_start,
+				 desc->debug_level);
+			  }
 			page_pop_page ();
 #if defined (METAHTML_PROFILER)
 			if (mhtml_profiling_functions &&
@@ -1273,7 +1299,10 @@ mhtml_find_user_function (char *name)
 {
   Symbol *sym = symbol_lookup_in_package (mhtml_user_keywords, name);
 
-  return (sym ? (UserFunction *)(sym->values) : (UserFunction *)NULL);
+  if ((sym != (Symbol *)NULL) && (sym->type == symtype_USERFUN))
+    return ((UserFunction *)(sym->values));
+
+  return ((UserFunction *)NULL);
 }
 
 /* Add or replace a function of TYPE with NAME, BODY in the
@@ -1561,7 +1590,8 @@ mhtml_let (PFunArgs)
       Symbol *sym = symbol_intern_in_package (pack, from->name);
 
       symbol_copy (sym, saved_vars);
-      symbol_copy (from, pack);
+      if (from != sym)
+	symbol_copy (from, pack);
     }
 
   /* Execute the body. */
@@ -1922,7 +1952,7 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 			 end, newstart, debug_level);
 	    }
 
-	  if (param_package)
+	  if (param_package && !param_package->name)
 	    symbol_destroy_package (param_package);
 	}
 
@@ -2206,7 +2236,8 @@ mhtml_all_digits (char *string)
 char *
 mhtml_base64decode (char *encoded, int *len)
 {
-  register int i, count;
+  register int i, count, c;
+  int newlines = 0;
   char *decoded;
   int decoded_size = 0;
   static int called_once = 0;
@@ -2231,8 +2262,13 @@ mhtml_base64decode (char *encoded, int *len)
 	transtab[(int) lut[i]] = (unsigned char) i;
    }
 
-  for (i = 0; transtab[(int)encoded[i]] < 64; i++);
-  count = i - 1;
+  for (i = 0; (c = transtab[(int)encoded[i]]) != -1; i++)
+    if ((encoded[i] != '\n') && (c >= 64))
+      break;
+    else if (encoded[i] == '\n')
+      newlines++;
+
+  count = (i - newlines) - 1;
   decoded_size = ((count + 3) / 4) * 3;
 
   decoded = (char *)xmalloc (1 + decoded_size);
@@ -2246,7 +2282,8 @@ mhtml_base64decode (char *encoded, int *len)
       decoded[i] = (DECODE (encoded[2]) << 6 | DECODE (encoded[3])); i++;
       encoded += 4;
       count -= 4;
-   }
+      while (*encoded == '\n') encoded++;
+    }
 
   i += ++count;
   decoded[i] = '\0';
@@ -2254,13 +2291,80 @@ mhtml_base64decode (char *encoded, int *len)
   return (decoded);
 }
 
+static void
+output64chunk (int c1, int c2, int c3, int pads, BPRINTF_BUFFER *buffer)
+{
+  static char basis_64[] =
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+
+  bprintf (buffer, "%c", basis_64[(c1 >> 2)]);
+  bprintf (buffer, "%c", basis_64[((c1 & 0x3) << 4) |
+				 ((c2 & 0xF0) >> 4)]);
+  if (pads == 2)
+    bprintf (buffer, "==");
+  else if (pads)
+    {
+      bprintf (buffer, "%c=", basis_64[((c2 & 0xF) << 2) |
+				      ((c3 & 0xC0) >> 6)]);
+    }
+  else
+    {
+      bprintf (buffer, "%c", basis_64[((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6)]);
+      bprintf (buffer, "%c", basis_64[c3 & 0x3F]);
+    }
+}
+
+char *
+mhtml_base64encode (char *data, int length, int shortlines)
+{
+  register int i;
+  BPRINTF_BUFFER *buffer = bprintf_create_buffer ();
+  char *result = (char *)NULL;
+  int c1, c2, c3, ct=0;
+
+  i = 0;
+  while (i < length)
+    {
+      c1 = data[i++];
+
+      if (i >= length)
+	{
+	  output64chunk (c1, 0, 0, 2, buffer);
+	}
+      else
+	{
+	  c2 = data[i++];
+
+	  if (i >= length)
+	    {
+	      output64chunk (c1, c2, 0, 1, buffer);
+	    }
+	  else
+	    {
+	      c3 = data[i++];
+	      output64chunk (c1, c2, c3, 0, buffer);
+	    }
+	}
+
+      ct += 4;
+      
+      if ((shortlines != 0) && ((ct % shortlines) == 0))
+	bprintf (buffer, "\n");
+    }
+
+  result = buffer->buffer;
+  free (buffer);
+  return (result);
+}
+
+
 int
 number_p (char *string)
 {
   register int i;
   int result = 0;
 
-  if (!string)
+  if (string == (char *)NULL)
     return (0);
 
   /* Skip leading whitespace. */
