@@ -49,8 +49,8 @@ typedef struct
   int type;		/* Either user_MACRO, user_SUBST, or user_DEFUN. */
   int flags;		/* Interesting bits about this function. */
   int debug_level;	/* How much debugging to do. */
-  char *name;		/* The name of this macro or subst. */
-  char *body;		/* The body of the macro or subst. */
+  char *name;		/* The name of this macro, function, or subst. */
+  char *body;		/* The body of this macro function, or subst. */
   char *packname;	/* Default package for the scope of this function. */
   char **named_parameters; /* Variables to bind during function invocation. */
   char **documentation;	/* The first set of comments that follow the defun. */
@@ -115,7 +115,7 @@ extern Package *mhtml_user_keywords;
 extern Package *PageVars;
 
 /* What a Meta-HTML builtin function takes as arguments:
-   PAGE is the entire page, as passed to page_fun_function ().
+   PAGE is the entire page, as passed to page_process_page_internal ().
    BODY is the contents of the tag body, exclusive of the tags.
    VARS is the list of variables found in the start tag.
    START is the absolute start of this tag in the page.
@@ -126,6 +126,7 @@ extern Package *PageVars;
                this function. */
 #define PFunArgs PAGE *page, PAGE *body, Package *vars, \
 		 int start, int end, int *newstart, int debug_level
+#define PassPFunArgs page, body, vars, start, end, newstart, debug_level
 
 typedef void PFunHandler (PFunArgs);
 
@@ -164,6 +165,13 @@ extern PageEnv *pagefunc_save_environment (void);
 extern void pagefunc_restore_environment (PageEnv *env);
 extern jmp_buf page_jmp_buffer;
 
+/* Variable specifically for the mdb debugger.  This allows the setting
+   of a callback function when an mdb breakpoint is hit. */
+typedef int DEBUGGER_CALLBACK_FUNCTION
+   (PFunArgs, PFunDesc *desc, UserFunction *uf, char *open_body);
+
+extern DEBUGGER_CALLBACK_FUNCTION *mhtml_debug_callback_function;
+
 #endif /* LIBMHTML_INTERNAL */
 
 #define user_MACRO 1
@@ -276,6 +284,7 @@ extern char *mhtml_funargs (Package *pack);
 extern int mhtml_all_digits (char *string);
 
 extern char *mhtml_base64decode (char *encoded, int *len);
+extern char *mhtml_base64encode (char *data, int length, int shortlines);
 
 extern PAGE *parser_top_page (void);
 extern int number_p (char *string);
