@@ -195,7 +195,9 @@ appear in the array.
 <li> <b>.lib</b>: A <meta-html> library file.
 </ul>
 
-<code>require</code> loads the newest version of the file that it finds, and records the complete pathname of the loaded file in the array variable <varref mhtml::require-loaded>.")
+<code>require</code> loads the newest version of the file that it
+finds, and records the complete pathname of the loaded file in the
+array variable <varref mhtml::require-loaded>.")
 {
   register int i;
   char *arg = mhtml_evaluate_string (get_positional_arg (vars, 0));
@@ -248,6 +250,8 @@ appear in the array.
 		bprintf_insert (file_contents, 0, "<*parser*::push-file %s>",
 				page_last_read_filename);
 		bprintf (file_contents, "<*parser*::pop-file>");
+		bprintf (file_contents, "<if <function-def .libinit> ");
+		bprintf (file_contents, "\"<.libinit><undef .libinit>\">");
 
 		/* Manually insert the file instead of letting bprintf
 		   do it for us.  This is because the file could contain
@@ -278,12 +282,35 @@ appear in the array.
 
 	    if (fd > -1)
 	      {
+		UserFunction *uf;
+
 		require_remember_pathname (pathname);
 		while ((pack = symbol_load_package (fd)) != (Package *)NULL);
+
 		if (mhtml_user_keywords == (Package *)NULL)
 		  mhtml_user_keywords = 
 		    symbol_lookup_package ("*user-functions*");
+
 		close (fd);
+
+		if ((uf = mhtml_find_user_function (".libinit"))
+		    != (UserFunction *)NULL)
+		  {
+		    char *open_body = strdup ("");
+		    PAGE *p = page_create_page ();
+		    Package *v = symbol_get_package ((char *)NULL);
+
+		    bprintf (p, " ");
+		    mhtml_execute_function
+		      (uf, p, p, v, start, end, newstart,
+		       uf->debug_level, open_body);
+
+		    symbol_destroy_package (v);
+		    page_free_page (p);
+		    free (open_body);
+		    open_body = mhtml_evaluate_string ("<undef .libinit>");
+		    xfree (open_body);
+		  }
 	      }
 	    else
 	      page_debug ("<REQUIRE %s> Failed in read of %s", arg, pathname);
