@@ -37,57 +37,19 @@
 /*							    */
 /************************************************************/
 
-/* <create-session VAR1 [... VARn] [ALT="value"] [allow-multiple]> */
 static void pf_create_session (PFunArgs);
-
-/* <delete-session> */
 static void pf_delete_session (PFunArgs);
-
-/* <set-session-var name=value> --> Stores in session database. */
 static void pf_set_session_var (PFunArgs);
-
-/* <get-session-var name> --> value from session database. */
 static void pf_get_session_var (PFunArgs);
-
-/* <unset-session-var name> Remove NAME from session database */
 static void pf_unset_session_var (PFunArgs);
-
-/* <set-session-timeout xxx> change default timeout value to xxx. */
 static void pf_set_session_timeout (PFunArgs);
-
-/* <require-session CODE> If invalid session ID, execute CODE. */
 static void pf_require_session (PFunArgs);
-
-/* <session-export package-name [session-db-name]>
-   --> Package symbols to session-db. */
 static void pf_session_export (PFunArgs);
-
-/* <session-import session-db-name [package-name]>
-   --> session-db symbols to specified package. */
 static void pf_session_import (PFunArgs);
-
-/* <with-locked-session> Per-session semaphore code </with-locked-session> */
 static void pf_with_locked_session (PFunArgs);
-
-/* <sessions-of-key key> --> Newline separated list of SIDs. */
 static void pf_sessions_of_key (PFunArgs);
-
-/* <get-session-data [sid]> --> Alist of the session data for SID. */
 static void pf_get_session_data (PFunArgs);
-
-/* <set-session-data [alist]> --> Set session info from ALIST. */
 static void pf_set_session_data (PFunArgs);
-
-#if defined (DEPRECATED_SESSION_COMMANDS)
-/* <session-export-posted "prefix"> --> posted items only to session-db. */
-static void pf_session_export_posted (PFunArgs);
-
-/* <session-import-posted "prefix"> --> prefix items to PageVars. */
-static void pf_session_import_posted (PFunArgs);
-#endif /* DEPRECATED_SESSION_COMMANDS */
-
-/* <set-session-db pathname> --> Dynamically change the location of the
-   file used to store session information. */
 static void pf_set_session_db (PFunArgs);
 
 static PFunDesc func_table[] =
@@ -111,11 +73,6 @@ static PFunDesc func_table[] =
   /* Deprecate this as soon as possible. */
   { "SESSION-DATA-ALIST", 0, 0, pf_get_session_data },
 
-#if defined (DEPRECATED_SESSION_COMMANDS)
-  { "SESSION-EXPORT-POSTED", 0, 0, pf_session_export_posted },
-  { "SESSION-IMPORT-POSTED", 0, 0, pf_session_import_posted },
-#endif /* DEPRECATED_SESSION_COMMANDS */
-
   { (char *)NULL,	0, 0, (PFunHandler *)NULL }
 };
 
@@ -136,8 +93,32 @@ DOC_SECTION (SESSION-OPERATORS)
      ALT="<set-var ERROR-MESSAGE=\"You are already signed on!\">
 		<include begin.html>"
    >  */
-static void
-pf_create_session (PFunArgs)
+DEFUN (pf_create_session,
+       &optional var=missing-code... &key alt=altcode allow-multiple,
+"Create a session, and place the session identifier in the variable
+<var default::sid>. The key for the session is created by
+concatenating the values of the <var VAR>s together, separated by
+hyphens (-). If a <var VAR> doesn't have a value currently assigned in
+the default package, but does have an optional <var MISSING-CODE>
+assignment, then the session is not created, and the <var
+MISSING-CODE> for that variable is evaluated. If all of the variables
+have values (or have no missing action), and the session could not be
+created, then <var ALTCODE> is evaluated if it is present.
+
+Finally, if the keyword <var ALLOW-MULTIPLE> is present, it indicates
+that the key does not have to be unique; it is enough that the session
+is created and a session ID returned.
+
+Example:
+<example>
+<create-session
+ USERNAME=<group <set-var ERROR-MESSAGE=\"You must supply a Username\">
+                 <include registration.html>>
+ PASSWORD=<group <set-var ERROR-MESSAGE=\"You must supply a Password\">
+                 <include registration.html>>
+      ALT=<group <set-var ERROR-MESSAGE=\"You are already signed on!\">
+                 <include begin.html>>>
+</example>")
 {
   register int i;
   BPRINTF_BUFFER *key_buffer = (BPRINTF_BUFFER *)NULL;
@@ -209,8 +190,11 @@ pf_create_session (PFunArgs)
     }
 }
 
-static void
-pf_delete_session (PFunArgs)
+DEFUN (pf_delete_session, ,
+"Immediately end the current session. This never returns anything. The
+current session is indicated by the value of the variable <var
+default::sid>, so you can delete sessions other than your own by
+setting that variable. Also see <funref SESSION-OPERATORS sessions-of-key>.")
 {
   session_id_t sid =
     (session_id_t)get_value (symbol_lookup_package ("DEFAULT"), "SID");
@@ -257,8 +241,12 @@ session_unlock (int lock, session_id_t sid)
     }
 }
 
-static void
-pf_with_locked_session (PFunArgs)
+DEFMACRO (pf_with_locked_session, ,
+"Executes <var body> in an environment where access to the current
+session is blocked. This allows concurrency (such as might be required
+when multiple <code>GET</code>s are being processed for a single user)
+to take place without introducing any race conditions for access to
+the current session record of the session database.")
 {
   session_id_t sid =
     (session_id_t)get_value (symbol_lookup_package ("DEFAULT"), "SID");
@@ -291,8 +279,17 @@ pf_with_locked_session (PFunArgs)
   if (jump_again) longjmp (page_jmp_buffer, 1);
 }
 
-static void
-pf_sessions_of_key (PFunArgs)
+DEFUN (pf_sessions_of_key, key,
+"Returns a newline separated list of the session ID's whose key
+matches <var key>.
+
+If <var key> is not specified, or if it contains only whitespace, all
+session ID's in the session database are returned. Since the list is
+newline separated, it is suitable for assignment to an array, as in:
+
+<example>
+<set-var sids[]=<sessions-of-key \"On-Line Shopping\">>
+</example>")
 {
   register int i;
   char *arg = mhtml_evaluate_string (get_positional_arg (vars, 0));
@@ -320,8 +317,9 @@ pf_sessions_of_key (PFunArgs)
   *newstart = start;
 }
       
-static void
-pf_set_session_var (PFunArgs)
+DEFUN (pf_set_session_var, name=value...,
+"Give the <var name>s the associated <var value>s in the session
+database record associated with the current session.")
 {
   session_id_t sid =
     (session_id_t)get_value (symbol_lookup_package ("DEFAULT"), "SID");
@@ -362,8 +360,34 @@ pf_set_session_var (PFunArgs)
     }
 }
 
-static void
-pf_get_session_var (PFunArgs)
+DEFUN (pf_get_session_var, &optional name...,
+"Return the value of the <var name>s given from the session data
+associated with the current session.  Each <var name> is a
+variable name which has had a value assigned to it with <funref
+SESSION-OPERATORS set-session-var>, or was created implicity via
+<funref SESSION-OPERATORS session-export>.
+
+The values are returned in the order in which the <var name>s appear.
+
+Examples:
+<example>
+<set-session-var foo=Var-1 bar=Var-2>
+<get-session-var foo>, <get-session-var bar>
+</example>
+produces
+<pre>
+<set-var foo=Var-1 bar=Var-2>
+<get-var foo>, <get-var bar>
+</pre>
+
+Example of retrieving a single session variable from a session
+package:
+<example>
+<set-var record::name=\"Brian Fox\">
+<set-var record::job=\"Programmer\">
+<session-export record record>
+<get-session-var record::name> --> Brian Fox
+</example>")
 {
   session_id_t sid =
     (session_id_t)get_value (symbol_lookup_package ("DEFAULT"), "SID");
@@ -403,8 +427,13 @@ pf_get_session_var (PFunArgs)
     }
 }
 
-static void
-pf_get_session_data (PFunArgs)
+/* <get-session-data [sid]> --> Alist of the session data for SID. */
+DEFUN (pf_get_session_data, &optional sid,
+"Returns an association list representing all of the data in the
+session database associated with <var sid>.
+
+If <var sid> is not supplied, it defaults to the value of <var
+default::sid> (i.e., the current session ID).")
 {
   char *passed = mhtml_evaluate_string (get_positional_arg (vars, 0));
   session_id_t sid;
@@ -436,13 +465,26 @@ valid_alist_p (char *alist)
   return (1);
 }
   
-static void
-pf_set_session_data (PFunArgs)
+/* <set-session-data [alist] [sid]> --> Set session info from ALIST. */
+DEFUN (pf_set_session_data, alist &optional sid,
+"Sets all of the session data associated with the session ID <var sid>
+to the contents of the association list <var alist>.  If <var sid> is
+not supplied, it defaults to the value of <var default::sid>.
+
+This construct is really of use only for those programs which deal
+directly with the session database, such as a backend tool for viewing
+and manipulating session data.  For all other applications, the use of
+<funref SESSION-OPERATORS set-session-var> and <funref
+SESSION-OPERATORS session-export> should be more than sufficient.")
 {
   char *alist = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  char *passed_sid = mhtml_evaluate_string (get_positional_arg (vars, 1));
   session_id_t sid;
 
-  sid = (session_id_t)get_value (symbol_lookup_package ("DEFAULT"), "SID");
+  if (!empty_string_p (passed_sid))
+    sid = (session_id_t)passed_sid;
+  else
+    sid = (session_id_t)get_value (symbol_lookup_package ("DEFAULT"), "SID");
 
   if ((sid != (session_id_t)0) && (!empty_string_p (alist)))
     {
@@ -469,11 +511,20 @@ pf_set_session_data (PFunArgs)
 	}
     }
 
-  if (alist != (char *)NULL) free (alist);
+  xfree (alist);
+  xfree (passed_sid);
 }
 
-static void
-pf_unset_session_var (PFunArgs)
+DEFUN (pf_unset_session_var, &optional name...,
+"Analogous to <funref VARIABLES unset-var>, <code>unset-session-var</code>
+removes the variables named by the given <var name>s from the session
+data associated with the current session.
+<example>
+<set-session-var foo=bar>
+<get-session-var foo>     --> bar
+<unset-session-var foo>
+<get-session-var foo>     -->
+</example>")
 {
   session_id_t sid =
     (session_id_t)get_value (symbol_lookup_package ("DEFAULT"), "SID");
@@ -510,8 +561,24 @@ pf_unset_session_var (PFunArgs)
     }
 }
 
-static void
-pf_session_export (PFunArgs)
+DEFUN (pf_session_export, package &optional to-package,
+"Save the package variables of <var package> in the current session.
+If <var to-package> is supplied, it indicates a new  package name to
+assign to each variable from <var package> that is saved.
+
+This function is quite useful when you wish to save all of the
+variables that were <b>posted</b> from the previous  URL (presumably,
+that URL contained a form whose <b>action</b> pointed to the current
+URL) in the session database record for the current session.  For
+example, if the previous URL posted the variables <code>FOO</code> and
+<code>BAR</code>, then executing:
+
+<example code><session-export posted blue></example>
+
+results in the session database containing values for
+<code>blue::FOO</code> and <code>blue::BAR</code>.  These values can
+either be looked up directly, or retrieved as a group (see <funref
+session-operators session-import> for more information.")
 {
   session_id_t sid =
     (session_id_t)get_value (symbol_lookup_package ("DEFAULT"), "SID");
@@ -584,8 +651,23 @@ pf_session_export (PFunArgs)
     }
 }
 
-static void
-pf_session_import (PFunArgs)
+DEFUN (pf_session_import, package &optional to-package,
+"Get the values of the session variables which belong to <var package>
+and set their values in the current package. If <var to-package> is
+supplied, it is the name of the package which will receive the
+variable values. Thus, if the session data for the current session
+contains:
+
+<example code>((BLUE::FOO . bar))</example>
+
+and then at a later time in the same session the command:
+
+<example code><session-import blue></example>
+
+is given, then the variable <code>FOO</code> in the current package
+will be set to <code>bar</code>.  This is extremely useful when
+bouncing back and forth between forms, see <funref sessions
+session-export>.")
 {
   SESSION_INFO *info = (SESSION_INFO *)NULL;
   session_id_t sid =
@@ -695,8 +777,12 @@ pf_session_import (PFunArgs)
 }
 
 
-static void
-pf_set_session_timeout (PFunArgs)
+DEFUN (pf_set_session_timeout, minutes,
+"Make the current session time out after <var minutes> minutes have
+transpired since the last access to that session.
+
+You cannot supply the value of (<code>0</code>); that is accomplished
+with <funref sessions delete-session>.")
 {
   char *timeout_ascii = mhtml_evaluate_string (get_positional_arg (vars, 0));
   session_id_t sid =
@@ -720,9 +806,54 @@ pf_set_session_timeout (PFunArgs)
     }
 }
 
-/* <require-session [missing/alt]="no-sid code" timeout="code"> */
-static void
-pf_require_session (PFunArgs)
+DEFUN (pf_require_session, &key missing=missing-code timeout=timeout-code,
+"Check that the current session is valid and not timed out.  If either
+of those things are true, the respective code is evaluated.  Usually,
+the targets of <var TIMEOUT> and <var MISSING> are a <funref
+FILE-OPERATORS redirect> command which points to a page explaining the
+problem.
+
+You could force an anonymous session with the following code:
+
+<example>
+  <set-var anonymous=\"Anonymous Session\">
+  <require-session
+    missing=\"<create-session anonymous allow-multiple>
+             <redirect <get-var SID>/<get-var mhtml::current-doc>\"
+    timeout=<redirect /timed-out.mhtml>>
+</example>
+
+In the real world, additional care should be taken when returning new
+URLs.  The variable <var mhtml::cookie-compatible> is
+<code>true</code> when the attached browser/server combination is
+capable of handling HTTP Cookies.  In this case, it is preferable to
+use the HTTP Cookie handling facilities instead of returning the
+session ID in the URL.  Here is how one might write the above example
+to handle both cases:
+
+<example>
+<set-var anonymous=\"Anonymous Session\" session-timeout=60>
+<require-session
+ missing = <prog
+             <create-session anonymous allow-multiple>
+             <set-session-timeout <get-var session-timeout>>
+             <when <not <get-var mhtml::cookie-compatible>>>
+             <redirect <concat
+                        <get-var mhtml::http-to-host>/
+                        <get-var SID><get-var mhtml::relative-prefix>/
+                        <get-var mhtml::current-doc>>>
+             </when>>
+ timeout = <prog
+             <set-var SID=\"\">
+             <if <get-var mhtml::cookie-compatible>
+                 <replace-page
+                  <get-var mhtml::relative-prefix>/timedout.mhtml>
+               <redirect
+                <get-var mhtml::url-to-dir-sans-sid>/timedout.mhtml>>>>
+</example>
+
+For more detail on the built-in variables provided by the <meta-html>
+Server and CGI Engine, see the section <secref server-variables>.")
 {
   char *missing_code = get_one_of (vars, "MISSING", "ALT", (char *)0);
   char *timeout_code = get_one_of (vars, "TIMEOUT", "TIMED-OUT", (char *)0);
@@ -745,6 +876,8 @@ pf_require_session (PFunArgs)
     bprintf_insert (page, start, "%s", code);
 }
 
+/* <set-session-db pathname> --> Dynamically change the location of the
+   file used to store session information. */
 static void
 pf_set_session_db (PFunArgs)
 {
@@ -761,24 +894,3 @@ pf_set_session_db (PFunArgs)
 
   xfree (session_db_file);
 }
-
-#if defined (DEPRECATED_SESSION_COMMANDS)
-static void
-pf_session_export_posted (PFunArgs)
-{
-  if (empty_string_p (body->buffer))
-    bprintf (body, "posted \"\"");
-  else
-    bprintf_insert (body, 0, "posted ");
-  vars = pagefunc_snarf_vars (body->buffer, 1);
-  pf_session_export (page, body, vars, start, end, newstart, debug_level);
-  symbol_destroy_package (vars);
-}
-
-static void
-pf_session_import_posted (PFunArgs)
-{
-  bprintf (body, " \"\"");
-  pf_session_import (page, body, vars, start, end, newstart, debug_level);
-}
-#endif /* DEPRECATED_SESSION_COMMANDS */
