@@ -55,7 +55,9 @@ static PFunDesc func_table[] =
 DOC_SECTION (STREAM-OPERATORS)
 PACKAGE_INITIALIZER (initialize_stream_functions)
 
-typedef enum { stream_FILE, stream_TCP, stream_CLOSED } StreamType;
+typedef enum
+  { stream_FILE, stream_TCP, stream_PROG, stream_CLOSED }
+StreamType;
 
 #define stream_MUST_SEEK 0x01
 #define stream_NOTIMEOUT 0x02
@@ -68,6 +70,8 @@ typedef struct
   char *name;		/* The name used to open this stream. */
   StreamType type;	/* The type of stream: stream_FILE, stream_TCP, etc. */
   int fd;		/* The file descriptor that describes this stream. */
+  int ifd;		/* For PROG pipes, the input channel to the stream. */
+  pid_t pid;		/* For PROG pipes, the pid of the child. */
   int mode;		/* Access mode. */
   int flags;		/* Special flags indicate changes in behaviour. */
   int last_op;		/* The last operation done on this stream. */
@@ -96,6 +100,7 @@ add_open_stream (char *name, StreamType type, int fd, int mode, int flags)
   stream->name = strdup (name);
   stream->type = type;
   stream->fd = fd;
+  stream->ifd = -1;
   stream->mode = mode;
   stream->flags = flags;
   stream->last_op = op_NONE;
@@ -129,6 +134,12 @@ discard_stream (int stream_index)
 	  (stream->fd != fileno (stderr)))
 	close (stream->fd);
 
+      if (stream->ifd != -1)
+	{
+	  close (stream->ifd);
+	  kill (stream->pid, SIGTERM);
+	}
+
       free (stream);
     }
 }
@@ -158,14 +169,70 @@ stream_type (char *typename)
 {
   StreamType type = stream_FILE;
 
-  if ((typename != (char *)NULL) && (strcasecmp (typename, "tcp") == 0))
-    type = stream_TCP;
+  if (typename != (char *)NULL)
+    {
+      if (strcasecmp (typename, "tcp") == 0)
+	type = stream_TCP;
+      else if (strcasecmp (typename, "prog") == 0)
+	type = stream_PROG;
+    }
 
   return (type);
 }
 
-/* <with-open-stream variable filename [type=[file | tcp]
-	   mode=[writer/reader/write-create]>
+static char **
+make_environ (void)
+{
+  register int i, j;
+  Symbol **envsyms = symbols_of_package (symbol_lookup_package ("ENV"));
+  Symbol *sym;
+  char **result = (char **)NULL;
+
+  if (envsyms != (Symbol **)NULL)
+    {
+      /* Find out how many symbols to make. */
+      for (i = 0, j = 0; (sym = envsyms[i]) != (Symbol *)NULL; i++)
+	if (sym->type == symtype_STRING)
+	  j++;
+
+      /* Make the resultant array the same size. */
+      result = (char **)xmalloc ((j + 1) * sizeof (char *));
+
+      /* For each symbol, make an environment variable. */
+      for (i = 0, j = 0; (sym = envsyms[i]) != (Symbol *)NULL; i++)
+	{
+	  if (sym->type == symtype_STRING)
+	    {
+	      char *value;
+
+	      if (sym->values && sym->values_index && sym->values[0])
+		value = sym->values[0];
+	      else
+		value = "";
+
+	      result[j] = (char *)xmalloc (3 + strlen (sym->name) +
+					   strlen (value));
+	      sprintf (result[j], "%s=%s", sym->name, value);
+	      j++;
+	    }
+	}
+      result[j] = (char *)NULL;
+    }
+
+  return (result);
+}
+
+/* Release a child that has died in the normal way. */
+static int child_status = 0;
+
+static void
+release_child (void)
+{
+  wait (&child_status);
+}
+
+/* <with-open-stream variable filename [type=[file|tcp|prog]
+	   mode=[write/reade/write-create]>
    [code using the open stream]
    </with-open-stream>
    Opens the file specified by FILENAME, and stores a referent to it
@@ -179,19 +246,22 @@ int mhtml_stdout_fileno = 1;
 int mhtml_stderr_fileno = 2;
 
 DEFMACRO (pf_with_open_stream,
-	  var name &key type=(FILE|TCP) mode=OPEN-MODE notimeout=true,
+	  var name &key type=(FILE|TCP|PROG) mode=OPEN-MODE notimeout=true,
 "Create an environment in which <var var> is bound to the
 <i>indicator</i> of an open stream, named by <var name>.
 
-The stream is either of type <code>FILE</code>, or <code>TCP</code>,
-in which case it is an open network connection.  When <var type> is
-<code>TCP</code>, <var name> specifies the host and port to connect
-to, in the form <code>Hostname:Port-Number</code>.  <var port-number>
-can either be a symbolic name which is widely recognized (e.g.,
-<code>SMTP</code>), or the digits which make up the port number,
-(e.g., <code>25</code>). Finally, the keyword argument <var
-notimeout=true> may be given, which indicates that the amount of time
-that <Meta-HTML> should wait during IO operations on this stream
+The stream is either of type <code>FILE</code>, in which case it is a simple
+file in the file system, of type <code>TCP</code>,
+in which case it is an open network connection, or of type <code>PROG</code>,
+in which case it is an open connection to a running process.
+
+When <var type> is <code>TCP</code>, <var name> specifies the host and
+port to connect to, in the form <code>Hostname:Port-Number</code>.
+<var port-number> can either be a symbolic name which is widely
+recognized (e.g., <code>SMTP</code>), or the digits which make up the
+port number, (e.g., <code>25</code>). Finally, the keyword argument
+<var notimeout=true> may be given, which indicates that the amount of
+time that <Meta-HTML> should wait during IO operations on this stream
 is infinite -- all processing will block until the stream is
 successfully read from or written to.  The default timeout is
 dependent on which specific operation is being performed, and the
@@ -241,6 +311,11 @@ created fresh, even if it had already existed.
       int flags = notimeout ? stream_NOTIMEOUT : 0;
       int stream_fd = -1;
 
+      /* For handling PROG streams. */
+      int stdin_pipe[2];
+      int stdout_pipe[2];
+      pid_t child = -1;
+
       xfree (modename);
       xfree (typename);
       xfree (notimeout);
@@ -273,20 +348,78 @@ created fresh, even if it had already existed.
 	    flags |= stream_MUST_SEEK;
 	  }
 	  break;
+
+	case stream_PROG:
+	  {
+	    pipe (stdin_pipe);
+	    pipe (stdout_pipe);
+	    child = vfork ();
+
+	    if (child != (pid_t) 0)
+	      {
+		/* In the parent. */
+		close (stdout_pipe[1]);
+		close (stdin_pipe[0]);
+
+		/* Say what to do when a child dies. */
+		signal (SIGCHLD, (sig_t)release_child);
+
+		/* Say what to do if the pipe is broken. */
+		signal (SIGPIPE, SIG_IGN);
+		stream_fd = stdin_pipe[1];
+	      }
+	    else
+	      {
+		/* In the child. */
+		char *argv[4];
+#if defined (__WINNT__)
+		char *physical_path = "/www/bin/bash.exe";
+#else
+		char *physical_path = "/bin/sh";
+#endif
+
+		close (stdout_pipe[0]);
+		close (stdin_pipe[1]);
+		dup2 (stdout_pipe[1], 1);
+		dup2 (stdin_pipe[0], 0);
+		dup2 (1, 2);
+		close (stdout_pipe[1]);
+		close (stdin_pipe[0]);
+
+		pagefunc_set_variable
+		  ("env::path", pagefunc_get_variable ("mhtml::exec-path"));
+		symbol_remove ("env::query_string");
+		symbol_remove ("env::path_info");
+		symbol_remove ("env::content_length");
+
+		argv[0] = physical_path;
+		argv[1] = "-c";
+		argv[2] = filename;
+		argv[3] = (char *)NULL;
+		execve (physical_path, argv, make_environ ());
+		_exit (127);
+	      }
+	  }
+	break;
 	}
 
       if (stream_fd > -1)
 	{
 	  char stream_value[40];
 	  PAGE *body_code = page_copy_page (body);
-	  int page_value;
-
-	  page_value =
+	  int page_value =
 	    add_open_stream (filename, type, stream_fd, mode, flags);
 
 	  sprintf (stream_value, "%d", page_value);
 	  pagefunc_set_variable (varname, stream_value);
 
+	  if (child > (pid_t) 0)
+	    {
+	      Stream *stream = open_streams[page_value];
+	      stream->ifd = stdout_pipe[0];
+	      stream->pid = child;
+	    }
+
 	  {
 	    PageEnv *environment = pagefunc_save_environment ();
 
@@ -314,7 +447,7 @@ created fresh, even if it had already existed.
 	}
       else
 	{
-	  char *temp = mhtml_funargs (vars);
+	  char *temp = mhtml_evaluate_string (mhtml_funargs (vars));
 
 	  page_syserr ("<with-open-stream%s%s>: %s",
 		       temp ? " " : "", temp ? temp : "",
@@ -395,7 +528,7 @@ set_io_timer (int length)
   if (divisor < 1)
     divisor = 512;
 
-  timeout = length / 512;
+  timeout = length / divisor;
   timeout += 10;
 
   operation_timed_out_p = 0;
@@ -450,9 +583,13 @@ stream_read (Stream *stream, char char_to_stop_at)
   char minbuf[512];
   int mindex = 0;
   int read_result = 0;
+  int input_fd = stream->fd;
+
+  if (stream->type == stream_PROG)
+    input_fd = stream->ifd;
 
   if ((stream->flags & stream_MUST_SEEK) && (stream->last_op != op_READ))
-    lseek (stream->fd, 0l, 1);
+    lseek (input_fd, 0l, 1);
 
   if (stream->flags & stream_NOTIMEOUT)
     set_io_timer (999999);
@@ -480,7 +617,7 @@ stream_read (Stream *stream, char char_to_stop_at)
 	      mindex = 0;
 	    }
 
-	  read_result = read (stream->fd, minbuf + mindex, 1);
+	  read_result = read (input_fd, minbuf + mindex, 1);
 	}
       else
 	{
@@ -508,7 +645,8 @@ stream_read (Stream *stream, char char_to_stop_at)
 		    }
 		}
 	    }
-	  else if (stream->type == stream_TCP)
+	  else if ((stream->type == stream_TCP) ||
+		   (stream->type == stream_PROG))
 	    {
 	      /* For TCP streams, we try hard to read in 10k chunks. */
 	      int bits_size = 20 * 1024;
@@ -522,7 +660,7 @@ stream_read (Stream *stream, char char_to_stop_at)
 		    }
 
 		  read_result =
-		    read (stream->fd, buffer + buffer_index, bits_size);
+		    read (input_fd, buffer + buffer_index, bits_size);
 
 		  if (read_result > 0)
 		    {
@@ -574,9 +712,13 @@ stream_read_chunk (Stream *stream, int length)
   int buffer_index = 0;
   int bytes_left = length;
   int done = 0;
+  int input_fd = stream->fd;
+
+  if (stream->type == stream_PROG)
+    input_fd = stream->ifd;
 
   if ((stream->flags & stream_MUST_SEEK) && (stream->last_op != op_READ))
-    lseek (stream->fd, 0l, 1);
+    lseek (input_fd, 0l, 1);
 
   buffer = (char *)xmalloc (length + 1);
 
@@ -593,7 +735,7 @@ stream_read_chunk (Stream *stream, int length)
 
   while (!done)
     {
-      int bytes_read = read (stream->fd, buffer + buffer_index, bytes_left);
+      int bytes_read = read (input_fd, buffer + buffer_index, bytes_left);
 
       if (bytes_read > 0)
 	{
@@ -692,7 +834,7 @@ available data is to supply the empty string as the value of <var stop-at>:
 
 <example>
 <with-open-stream stream www.ua.com:80 type=tcp mode=read-write>
-  <stream-put stream \"GET /welcome.mhtml HTTP/1.0\n\n\">
+  <stream-put stream \"GET /welcome.mhtml HTTP/1.0\\n\\n\">
   <set-var the-page = <stream-get stream stop-at=\"\">>
 </with-open-stream>
 </example>")
@@ -833,12 +975,16 @@ no waiting period is used.")
 	fd_set read_fds;
 	struct timeval poll;
 	int ready;
+	int input_fd = stream->fd;
+
+	if (stream->type == stream_PROG)
+	  input_fd = stream->ifd;
 
 	FD_ZERO (&read_fds);
-	FD_SET (stream->fd, &read_fds);
+	FD_SET (input_fd, &read_fds);
 	poll.tv_sec = delay_in_seconds;
 	poll.tv_usec = 0;
-	ready = select (stream->fd + 1, &read_fds, (fd_set *)0, (fd_set *)0,
+	ready = select (input_fd + 1, &read_fds, (fd_set *)0, (fd_set *)0,
 			&poll);
 
 	if (ready < 1)
