@@ -50,6 +50,13 @@ static void pf_word_wrap (PFunArgs);
 static void pf_pad (PFunArgs);
 static void pf_string_eq (PFunArgs);
 static void pf_plain_text (PFunArgs);
+static void pf_base64decode (PFunArgs);
+static void pf_base64encode (PFunArgs);
+
+#if defined (HAVE_CRYPT)
+static void pf_unix_crypt (PFunArgs);
+#endif
+
   /* Random string operations. */
 static PFunDesc func_table[] =
 {
@@ -66,6 +73,11 @@ static PFunDesc func_table[] =
   { "PAD",		0, 0, pf_pad },
   { "STRING-EQ",	0, 0, pf_string_eq },
   { "PLAIN-TEXT",	1, 0, pf_plain_text },
+  { "BASE64DECODE",	0, 0, pf_base64decode },
+  { "BASE64ENCODE",	0, 0, pf_base64encode },
+#if defined (HAVE_CRYPT)
+  { "UNIX::CRYPT",	0, 0, pf_unix_crypt },
+#endif
 
   { (char *)NULL,	0, 0, (PFunHandler *)NULL }
 };
@@ -83,7 +95,7 @@ DEFUN (pf_string_length, string,
   char *string = mhtml_evaluate_string (get_positional_arg (vars, 0));
   int length = 0;
 
-  if (!empty_string_p (string))
+  if (string != (char *)NULL)
     length = strlen (string);
 
   xfree (string);
@@ -136,7 +148,7 @@ Here are a few examples:
   <match \"foobar\" \"oob\" action=startpos>--> \"1\"
   <match \"foobar\" \"oob\" action=endpos>  --> \"4\"
   <match \"foobar\" \"oob\" action=length>  --> \"3\"
-  <match \"foobar\" \"[0-9]*\">             --> \"\"
+  <match \"foobar\" \"[0-9]+\">             --> \"\"
 </example>")
 {
   char *_string = get_positional_arg (vars, 0);
@@ -415,13 +427,17 @@ the output of <funref osfuncs cgi-exec>.")
   if (!empty_string_p (varname))
     {
       char *contents = pagefunc_get_variable (varname);
-      char *result = subst_in_string_internal (contents, vars, debug_level);
 
-      pagefunc_set_variable (varname, result);
-      if (result) free (result);
+      if (contents != (char *)NULL)
+	{
+	  char *result = subst_in_string_internal
+	    (contents, vars, debug_level);
+
+	  pagefunc_set_variable (varname, result);
+	  xfree (result);
+	}
     }
-
-  if (varname != (char *)NULL) free (varname);
+  xfree (varname);
 }
 
 DEFUN (pf_subst_in_string, string &rest regexp replacement,
@@ -464,7 +480,7 @@ lowercase.
 <downcase \"This is Written in Meta-HTML\">
 </complete-example>")
 {
-  char *value = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  unsigned char *value = mhtml_evaluate_string (get_positional_arg (vars, 0));
 
   if (value != (char *)NULL)
     {
@@ -488,7 +504,7 @@ uppercase.
 <upcase \"This is Written in Meta-HTML\">
 </complete-example>")
 {
-  char *value = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  unsigned char *value = mhtml_evaluate_string (get_positional_arg (vars, 0));
 
   if (value != (char *)NULL)
     {
@@ -515,7 +531,7 @@ lowercase depending on the surrounding characters.
 Also see <funref string-operators downcase>, and
 <funref string-operators upcase>.")
 {
-  char *value = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  unsigned char *value = mhtml_evaluate_string (get_positional_arg (vars, 0));
 
   if (value != (char *)NULL)
     {
@@ -524,6 +540,9 @@ Also see <funref string-operators downcase>, and
 
       for (i = 0; value[i] != '\0'; i++)
 	{
+	  if (value[i] > 127)
+	    continue;
+
 	  if (!isalpha (value[i]))
 	    capnext = 1;
 	  else
@@ -832,6 +851,7 @@ Examples:
   if (input)
     {
       bprintf_insert (page, start, "%s", input);
+      *newstart += strlen (input);
       free (input);
     }
 }
@@ -1009,3 +1029,234 @@ This is line 2.
   bprintf_free_buffer (evalbody);
 }
 
+DEFUN (pf_base64encode, varname &key shortlines=[true|value],
+"Performs the translation operation commonly known as <i>Base64
+Encoding</i> on the contents of the binary variable referenced by <var
+varname>, and returns the results of that encoding.
+
+Base64 encoding is a common transfer encoding for binary data and for
+Basic Authorization values -- this function can be used to turn data
+from its original pre-encoded state to an encoded one.
+
+If the keyword argument <var shortlines=value> is supplied, then the result
+of encoding is broken up into lines containing <var value> characters
+each, rounded up to the nearest multiple of 4.  This format is
+commonly implemented by many base 64 encoding programs.  A value of
+<code>true</code> uses the Meta-HTML default value of 64.  If the
+argument is not supplied, the data is returned in a single line of data.
+
+The following code reads a GIF image into a binary variable, and then
+displays the base64 encoded version:
+<example>
+<with-open-stream s /tmp/image.gif mode=read>
+  <stream-get-contents s gif>
+</with-open-stream>
+<base64encode gif>
+</example>")
+{
+  char *varname = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  char *result = (char *)NULL;
+  char *sl_arg = mhtml_evaluate_string
+    (get_one_of (vars, "shortlines", "short-lines", (char *)NULL));
+  int shortlines = 0;
+
+  if (!empty_string_p (sl_arg))
+    {
+      shortlines = atoi (sl_arg);
+      if (!shortlines) shortlines = 64;
+
+      shortlines = ((shortlines + 3) / 4) * 4;
+    }
+
+  xfree (sl_arg);
+
+  if (!empty_string_p (varname))
+    {
+      Symbol *sym = symbol_lookup (varname);
+
+      if (sym != (Symbol *)NULL)
+	{ 
+	  switch (sym->type)
+	    {
+	    case symtype_BINARY:
+	      {
+		Datablock *block = (Datablock *)(sym->values);
+		result = mhtml_base64encode (block->data,
+					     block->length, shortlines);
+	      }
+	    break;
+
+	    case symtype_STRING:
+	      {
+		char *string = pagefunc_get_variable (varname);
+
+		if (string != (char *)NULL)
+		  {
+		    int length = strlen (string);
+		    result = mhtml_base64encode (string, length, shortlines);
+		  }
+	      }
+	    break;
+	    }
+	}
+    }
+
+  xfree (varname);
+
+  if (result)
+    {
+      bprintf_insert (page, start, "%s", result);
+      *newstart += strlen (result);
+      free (result);
+    }
+}
+
+DEFUN (pf_base64decode, string &optional varname,
+"Performs the translation operation commonly known as <i>Base64
+Decoding</i> on <var STRING>, and returns the results of that
+decoding.
+
+If the optional <var varname> is supplied, the results of decoding are
+placed into the binary variable named <var varname> instead of being
+returned in the page.  This allows random binary data to be decoded,
+and perhaps written to the output stream using <funref
+stream-operators stream-put-contents>.
+
+Base64 encoding is a common transfer encoding for binary data and for
+Basic Authorization values -- this function can be used to turn such
+strings into their original, pre-encoded state.
+
+<complete-example>
+<set-var the-data = \"YmZveDpmcm9ibml0eg==\">
+<base64decode <get-var the-data>>
+</complete-example>")
+{
+  char *string = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  
+  if (!empty_string_p (string))
+    {
+      int length = 0;
+      char *result = mhtml_base64decode (string, &length);
+      char *varname = mhtml_evaluate_string (get_positional_arg (vars, 1));
+
+      if (empty_string_p (varname))
+	{
+	  /* Manually insert the data instead of letting bprintf
+	     do it for us.  This is because the data could contain
+	     null characters, and then the buffer wouldn't necessarily
+	     reflect the length of what was inserted. */
+	  if ((length + page->bindex) >= page->bsize)
+	    page->buffer = (char *)xrealloc
+	      (page->buffer, (page->bsize += (length + 100)));
+
+	  memmove (page->buffer + start + length, page->buffer + start,
+		   (page->bindex + 1) - start);
+
+	  memcpy (page->buffer + start, result, length);
+	  page->bindex += length;
+	  *newstart += length;
+	}
+      else
+	{
+	  Symbol *sym = symbol_remove (varname);
+	  Datablock *block;
+
+	  symbol_free (sym);
+
+	  sym = symbol_intern (varname);
+	  block = datablock_create (result, length);
+	  sym->type = symtype_BINARY;
+	  sym->values = (char **)block;
+	}
+
+      xfree (varname);
+      xfree (result);
+    }
+
+  xfree (string);
+}
+
+#if defined (HAVE_CRYPT)
+/* Create a password from cleartext. */
+static char *
+create_password (char *clear, char *salt)
+{
+  int length = (13 * ((strlen (clear) + 7) / 8));
+  char *encrypted = (char *)xmalloc (1 + length);
+  char *clear_p = clear;
+
+  encrypted[0] = '\0';
+
+  while (length > 0)
+    {
+      char chunk[9];
+      char *temp;
+
+      strncpy (chunk, clear_p, 8);
+      chunk[8] = (char)0;
+
+      temp = crypt (chunk, salt);
+      strcat (encrypted, temp);
+
+      clear_p += 8;
+      length -= 13;
+    }
+
+  return (encrypted);
+}
+
+DEFUNX (pf_unix::crypt, string &optional salt,
+"Return <var string> encrypted using the local system's <code>crypt()</code>
+function with the salt <var salt>.
+
+<var salt> is a two character string -- if you wish to compare the
+cleartext value that a user has entered against a Unix-style encrypted
+password, such as one from <code>/etc/passwd</code>, or from a
+<code>.htaccess</code> file, use the first two characters of the
+existing encrypted password as the salt, and then compare the
+resulting strings.
+
+For example, if the variable <var existing-pass> contains a previously
+encypted password, and the variable <var entered-pass> contains the
+cleartext that the user has just entered, you may encrypt the user's
+password and compare it with the existing one with the following code:
+
+<example>
+<set-var encrypted-pass =
+   <unix::crypt <get-var entered-pass>
+                <substring <get-var existing-pass> 0 2>>>
+.blank
+<when <string-eq <get-var encrypted-pass>
+                 <get-var existing-pass>>>
+   <set-session-var logged-in=true>
+   <redirect members-only.mhtml>
+</when>
+.blank
+<h3>Please enter your password again.  It didn't match.</h3>
+</example>")
+
+static void
+pf_unix_crypt (PFunArgs)
+{
+  char *string = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  char *saltarg = mhtml_evaluate_string (get_positional_arg (vars, 1));
+  char salt[3] = { 'c', 'd', '\0' }; /* Anything, right? */
+
+  if (!empty_string_p (saltarg))
+    {
+      salt[0] = saltarg[0];
+      if (saltarg[1]) salt[1] = saltarg[1];
+    }
+
+  if (!empty_string_p (string))
+    {
+      char *insertion = create_password (string, salt);
+      bprintf_insert (page, start, "%s", insertion);
+      *newstart += strlen (insertion);
+      free (insertion);
+    }
+
+  xfree (string);
+  xfree (saltarg);
+}
+#endif /* HAVE_CRYPT */
