@@ -203,6 +203,47 @@ mhtml::relative-prefix> among others.")
   generic_set_variable (vars, debug_level, 1, 1);
 }
 
+static void
+get_var_internal (PFunArgs, int once)
+  {
+  register int i;
+  char *name;
+
+  for (i = 0; (name = get_positional_arg (vars, i)) != (char *)NULL; i++)
+    {
+      char *insertion;
+      char *value;
+
+      insertion = mhtml_evaluate_string (name);
+      value = pagefunc_get_variable (insertion);
+
+      if (debug_level > 5)
+	page_debug ("<get-var \"%s\">", insertion ? insertion : "");
+
+      if (value)
+	{
+	  int len = strlen (value);
+	  bprintf_insert_binary (page, start, value, len);
+	  start += len;
+	}
+      else
+	{
+	  if (debug_level > 9)
+	    page_debug ("<get-var \"%s\">: Unbound Variable \"%s\"!",
+			insertion, insertion);
+	}
+
+      if (debug_level > 5)
+	page_debug ("--> `%s'", value ? value : "");
+
+      if (insertion)
+	free (insertion);
+
+      if (once)
+	*newstart = start;
+    }
+}
+
 DEFUN (pf_get_var, &optional name...,
 "Return the value of the <var name>s given.  Each <var name> is a
 variable name which has had a value assigned to it with <funref
@@ -223,39 +264,7 @@ When multiple <var name>s are given:
 <get-var foo bar foo>
 </complete-example>")
 {
-  register int i;
-  char *name;
-
-  for (i = 0; (name = get_positional_arg (vars, i)) != (char *)NULL; i++)
-    {
-      char *insertion;
-      char *value;
-
-      insertion = mhtml_evaluate_string (name);
-
-      value = pagefunc_get_variable (insertion);
-
-      if (debug_level > 5)
-	page_debug ("<get-var \"%s\">", insertion ? insertion : "");
-
-      if (value)
-	{
-	  bprintf_insert (page, start, "%s", value);
-	  start += strlen (value);
-	}
-      else
-	{
-	  if (debug_level > 10)
-	    page_debug ("<get-var \"%s\">: Unbound Variable \"%s\"!",
-			insertion, insertion);
-	}
-
-      if (debug_level > 5)
-	page_debug ("--> `%s'", value ? value : "");
-
-      if (insertion)
-	free (insertion);
-    }
+  get_var_internal (PassPFunArgs, 0);
 }
 
 DEFUN (pf_get_var_once, &optional name...,
@@ -274,41 +283,7 @@ but...
 <get-var foo>        --> HELLO
 </example>")
 {
-  register int i;
-  char *name;
-
-  for (i = 0; (name = get_positional_arg (vars, i)) != (char *)NULL; i++)
-    {
-      char *insertion;
-      char *value;
-
-      insertion = mhtml_evaluate_string (name);
-
-      value = pagefunc_get_variable (insertion);
-
-      if (debug_level > 5)
-	page_debug ("<get-var \"%s\">", insertion ? insertion : "");
-
-      if (value)
-	{
-	  bprintf_insert (page, start, "%s", value);
-	  start += strlen (value);
-	}
-      else
-	{
-	  if (debug_level > 10)
-	    page_debug ("<get-var \"%s\">: Unbound Variable \"%s\"!",
-			insertion, insertion);
-	}
-
-      if (debug_level > 5)
-	page_debug ("--> `%s'", value ? value : "");
-
-      if (insertion)
-	free (insertion);
-
-      *newstart = start;
-    }
+  get_var_internal (PassPFunArgs, 1);
 }
 
 DEFUN (pf_unset_var, &optional name...,
@@ -442,7 +417,7 @@ DEFUN (pf_increment, name &key by=amount,
 
 Also see <funref variables decrement>.")
 {
-  change_increment (page, body, vars, start, end, newstart, debug_level, 1);
+  change_increment (PassPFunArgs, 1);
 }
 
 DEFUN (pf_decrement, name &key by=amount,
@@ -457,10 +432,10 @@ DEFUN (pf_decrement, name &key by=amount,
 
 Also see <funref variables increment>.")
 {
-  change_increment (page, body, vars, start, end, newstart, debug_level, -1);
+  change_increment (PassPFunArgs, -1);
 }
 
-DEFUN (pf_cgi_encode, &rest vars &key preserve-case=true,
+DEFUN (pf_cgi_encode, &rest vars &key preserve-case=true strip=true,
 "A CGI readable string is created from the names of the <var var>s given,
 and the associated values of those variables.  For example, if the
 variable <code>FOO-VAR</code> has the value <code>\"Foo &
@@ -474,9 +449,17 @@ Given the optional keyword argument of <var preserve-case=true>,
 <code>cgi-encode</code> encodes the variables preserving the case of
 them as they were input.
 
-Thus,
 <complete-example>
 <cgi-encode Foo-Var preserve-case=true>
+</complete-example>
+
+Finally, the keyword flag <var strip=true>, when present, says to
+strip off the package name of each variable before placing it in an
+assignment statement (also see <funref packages package-vars>).
+
+<complete-example>
+<set-var FOO::BAR=value>
+<cgi-encode Foo::Bar preserve-case=true strip=true>
 </complete-example>")
 {
   char **names = get_vars_names (vars);
@@ -489,10 +472,13 @@ Thus,
       Package *cgivars = symbol_get_package ((char *)NULL);
       Symbol **symbols = (Symbol **)NULL;
       int save_case_p = 0;
+      int strip_package = 0;
 
       {
 	char *temp = mhtml_evaluate_string (get_value (vars, "preserve-case"));
 	if (!empty_string_p (temp)) save_case_p++;
+	temp = mhtml_evaluate_string (get_value (vars, "strip"));
+	if (!empty_string_p (temp)) strip_package++;
       }
 
       for (i = 0; (name = names[i]) != (char *)NULL; i++)
@@ -505,7 +491,20 @@ Thus,
 	      if ((sym != (Symbol *)NULL) && (sym->type == symtype_STRING))
 		{
 		  register int j;
-		  Symbol *newsym = symbol_intern_in_package (cgivars, name);
+		  Symbol *newsym;
+
+		  if (strip_package)
+		    {
+		      char *tempname = strstr (name, "::");
+		      if (tempname)
+			{
+			  tempname = strdup (tempname + 2);
+			  free (name);
+			  name = tempname;
+			}
+		    }
+
+		  newsym = symbol_intern_in_package (cgivars, name);
 
 		  if (save_case_p)
 		    newsym->preserved_name = strdup (name);
@@ -515,7 +514,7 @@ Thus,
 		}
 	    }
 
-	  if (name) free (name);
+	  xfree (name);
 	}
 
       symbols = symbols_of_package (cgivars);
