@@ -1305,10 +1305,12 @@ mhttpd_set_mhtml_variables (HTTP_RESULT *result)
 
   pagefunc_set_variable ("mhtml::version", sv_MHTML_VERSION);
 
-  if (port != 80)
-    bprintf (buffer, "%s://%s:%d", protocol, server_name, port);
-  else
+  if (port == 80)
     bprintf (buffer, "%s://%s", protocol, server_name);
+  else if (port == 443)
+    bprintf (buffer, "https://%s", server_name);
+  else
+    bprintf (buffer, "%s://%s:%d", protocol, server_name, port);
 
   http_prefix = buffer->buffer;
   buffer->buffer = (char *)NULL;
@@ -1518,7 +1520,7 @@ decompose_and_post (char *enctype, char *data, int length)
   register int i;
   char *boundary = (char *)NULL;
   Package *package;
-
+      
   if (!length)
     return;
 
@@ -1571,233 +1573,285 @@ decompose_and_post (char *enctype, char *data, int length)
 	{
 	  if (strncmp (data + i, boundary, blen) == 0)
 	    {
-	      char *name = (char *)NULL;
-	      char *filename = (char *)NULL;
-	      char *value = (char *)NULL;
-	      int value_len = 0;
-	      int quoted = 0;
 	      int start, end;
+	      Package *temp_pack = symbol_get_package ((char *)NULL);
+	      char *posted_data = (char *)NULL;
+	      int posted_data_len = 0;
 
-	      /* Found one. Skip whitespace and read the content-disposition
-		 information. */
-	      i += blen;
-	      while (word_separator (data[i])) i++;
+	      /* Found one. If it is the last one, quit now.  Otherwise,
+		 skip whitespace and decode the MIME data that follows. */
+	      i += blen; if (i >= limit) break;
+	      while ((i < limit) && (word_separator (data[i]))) i++;
 
-	      /* We are simple-minded today, and are insisting on the next
-		 string being Content-Disposition. */
-	      if (strncasecmp (data + i, "Content-Disposition:", 20) != 0)
+	      /* Next come a set of headers.  Bind them to their values
+		 in a local package. */
+	      while (1)
 		{
-		  mhttpd_log (log_DEBUG, "Missing Content-Disposition");
-		  continue;
-		}
+		  char header_title[1024];
+		  char header_value[1024];
 
-	      i += 20;
-	      while (whitespace (data[i])) i++;
+		  header_title[0] = '\0';
+		  header_value[0] = '\0';
 
-	      /* The next item HAS to be "form-data;". */
-	      if (strncasecmp (data + i, "form-data;", 10) != 0)
-		{
-		  mhttpd_log (log_DEBUG, "Missing `form-data;'");
-		  continue;
-		}
-
-	      i += 10;
-	      while (whitespace (data[i])) i++;
-
-	      /* Now we are expecting the name of the variable to bind. */
-	      if (strncasecmp (data + i, "name=", 5) != 0)
-		{
-		  mhttpd_log (log_DEBUG, "Missing `name='");
-		  continue;
-		}
-
-	      i += 5;
-	      while (whitespace (data[i])) i++;
-
-	      /* Okay, read the name of this variable. */
-	      if (data[i] == '"')
-		{
-		  quoted = 1;
-		  i++;
-		}
-
-	      start = end = i;
-	      while (data[i])
-		{
-		  if (quoted && data[i] == '"')
+		  /* At end of header information? */
+		  if (((data[i] == '\r') && (data[i + 1] == '\n')) ||
+		      (data[i] == '\n'))
 		    {
-		      end = i;
-		      i++;
-		      quoted = 0;
+		      if (data[i] == '\r') i++;
+		      if (data[i] == '\n') i++;
 		      break;
 		    }
 
-		  if (!quoted && (whitespace (data[i]) || data[i] == ';'))
+		  /* Find a header. */
+		  start = i;
+		  while ((data[i] != ':') && (!word_separator (data[i]))) i++;
+		  end = i;
+		  if ((end - start) < 1023)
 		    {
+		      strncpy (header_title, data + start, end - start);
+		      header_title[end - start] = '\0';
+
+		      /* Now find the value for this header. */
+		      /* Skip the ":" which ended this title and any
+			 following whitespace characters. */
+		      if (data[i]) i++;
+		      while ((i < limit) && (word_separator (data[i]))) i++;
+		      start = i;
+		      while ((i < limit) && (data[i] != '\n')) i++;
 		      end = i;
-		      break;
+		      if (data[end - 1] == '\r') end--;
+
+		      if ((end - start) < 1023)
+			{
+			  int l = end - start;
+			  strncpy (header_value, data + start, l);
+			  header_value[l] = '\0';
+			  l--;
+			  while (whitespace (header_value[l])) l--;
+			  header_value[++l] = '\0';
+			}
 		    }
 
-		  i++;
+		  /* At any rate, skip to the end of this line. */
+		  while ((i < limit) && (data[i] != '\n')) i++;
+		  if (data[i] == '\n') i++;
+
+		  /* If we have both a header and a value, add them to this
+		     package. */
+		  if ((header_title[0] != '\0') && (header_value[0] != '\0'))
+		    {
+		      Symbol *sym = symbol_intern_in_package
+			(temp_pack, header_title);
+		      symbol_add_value (sym, header_value);
+		    }
 		}
 
-	      name = (char *)xmalloc (1 + (end - start));
-	      strncpy (name, data + start, (end - start));
-	      name[end - start] = '\0';
+	      /* We have gobbled up all of the headers into our local
+		 package, and the data pointer is at the start of the
+		 data for this MIME entry.  Advance to the end of that
+		 data, and find out what to do with it. */
+	      start = i;
+	      while ((i < limit) && (strncmp (data + i, boundary, blen) != 0))
+		i++;
+	      end = i;
 
-	      /* We have the name of the variable.  Optionally, this
-		 is a file, so save the filename if present. */
-	      if (data[i] == ';')
+	      /* Now, it might be me, but it appears that the fucking
+		 Netscape Browser has a different view of the boundary
+		 string in the Content-Encoding header than it places in
+		 the body.  This is really heinous, but we try to
+		 handle it here. */
+	      while (data[end - 1] == '-') end--;
+
+	      if (data[end - 1] == '\n')
 		{
-		  i++;
-		  while (whitespace (data[i])) i++;
+		  --end;
+		  if (data[end - 1] == '\r')
+		    --end;
+		}
 
-		  if (strncasecmp (data +i, "filename=", 9) == 0)
+	      /* The data corresponding to this item is delimited by the
+		 indices in START and END.  The package TEMP_PACK contains
+		 symbols whose names are MIME headers, and whose values
+		 are the corresponding value strings.  We can only
+		 handle the item if it contains a
+		 Content-Disposition header. */
+	      {
+		Symbol *sym = symbol_lookup_in_package
+		  (temp_pack, "Content-Disposition");
+		char *value = (char *)NULL;
+
+		if ((sym != (Symbol *)NULL) && (sym->values != (char **)NULL))
+		  value = sym->values[0];
+
+		if (value == (char *)NULL)
+		  {
+		    mhttpd_log (log_DEBUG, "Missing Content-Disposition!");
+		    continue;
+		  }
+
+		/* We only know how to handle FORM-DATA types. */
+		if (strncasecmp (value, "form-data;", 10) != 0)
+		  {
+		    mhttpd_log (log_DEBUG, "Missing `form-data;'");
+		    continue;
+		  }
+
+		/* Now, find the name. */
+		value += 10;
+		while (whitespace (*value)) value++;
+
+		/* Now we are expecting the name of the variable to bind. */
+		if (strncasecmp (value, "name=", 5) != 0)
+		  {
+		    mhttpd_log (log_DEBUG,
+				"Missing `name=' in Content-Disposition");
+		    continue;
+		  }
+
+		value += 5;
+		while (whitespace (*value)) value++;
+
+		/* Okay, read the name of this variable. */
+		{
+		  register int j = 0;
+		  char *name = (char *)NULL;
+		  char *filename = (char *)NULL;
+		  int nstart = 0;
+		  int nend = 0;
+		  int quoted = 0;
+
+		  if (value[j] == '"')
 		    {
-		      i += 9;
+		      quoted = 1;
+		      j++;
+		    }
 
-		      /* Okay, read the name of this file. */
-		      if (data[i] == '"')
+		  nstart = nend = j;
+		  while (value[j])
+		    {
+		      if (quoted && value[j] == '"')
 			{
-			  quoted = 1;
-			  i++;
+			  nend = j;
+			  j++;
+			  quoted = 0;
+			  break;
 			}
 
-		      start = end = i;
-		      while (data[i])
+		      if (!quoted &&
+			  (whitespace (value[j]) || value[j] == ';'))
 			{
-			  if (quoted && data[i] == '"')
-			    {
-			      end = i;
-			      i++;
-			      quoted = 0;
-			      break;
-			    }
-
-			  if (!quoted && whitespace (data[i]))
-			    {
-			      end = i;
-			      break;
-			    }
-			  i++;
+			  nend = j;
+			  break;
 			}
 
-		      filename = (char *)xmalloc (1 + (end - start));
-		      strncpy (filename, data + start, (end - start));
-		      filename[end - start] = '\0';
+		      j++;
+		    }
+
+		  name = (char *)xmalloc (1 + (nend - nstart));
+		  strncpy (name, value + nstart, (nend - nstart));
+		  name[nend - nstart] = '\0';
+
+		  /* We have the name of the variable.  Optionally, this
+		     is a file, so save the filename if present. */
+		  if (value[j] == ';')
+		    {
+		      j++;
+		      while (whitespace (value[j])) j++;
+
+		      if (strncasecmp (value + j, "filename=", 9) == 0)
+			{
+			  j += 9;
+
+			  /* Okay, read the name of this file. */
+			  if (value[j] == '"')
+			    {
+			      quoted = 1;
+			      j++;
+			    }
+
+			  nstart = nend = j;
+			  while (value[j])
+			    {
+			      if (quoted && value[j] == '"')
+				{
+				  nend = j;
+				  j++;
+				  quoted = 0;
+				  break;
+				}
+
+			      if (!quoted && whitespace (value[j]))
+				{
+				  nend = j;
+				  break;
+				}
+			      j++;
+			    }
+
+			  filename = (char *)xmalloc (1 + (nend - nstart));
+			  strncpy (filename, value + nstart, (nend - nstart));
+			  filename[nend - nstart] = '\0';
+			}
+		    }
+
+		  posted_data_len = (end - start);
+		  posted_data = (char *)xmalloc (1 + posted_data_len);
+		  memcpy (posted_data, data + start, posted_data_len);
+		  posted_data[posted_data_len] = '\0';
+
+		  /* Save value and name in the posted package. */
+		  if (filename != (char *)NULL)
+		    {
+		      Datablock *block =
+			datablock_create (posted_data, posted_data_len);
+
+		      /* Kill old definition if there is one. */
+		      sym = symbol_remove_in_package (package, name);
+		      symbol_free (sym);
+
+		      /* Create new definition. */
+		      sym = symbol_intern_in_package (package, name);
+		      sym->type = symtype_BINARY;
+		      sym->values = (char **)block;
+
+		      /* Create a new variable called NAME-FILENAME. */
+		      {
+			char *newname = (char *)xmalloc (40 + strlen (name));
+			sprintf (newname, "POSTED::%s-FILENAME", name);
+			pagefunc_set_variable (newname, filename);
+
+			/* Create a new variable called NAME-TYPE. */
+			sprintf (newname, "POSTED::%s-TYPE", name);
+			pagefunc_set_variable
+			  (newname, forms_get_tag_value_in_package
+			   (temp_pack, "Content-Type"));
+			free (newname);
+		      }
 		    }
 		  else
 		    {
-		      /* Skip to first whitespace character. */
-		      while (!whitespace (data[i])) i++;
+		      /* Just a regular variable.  Add this name/value to the
+			 POSTED package. */
+		      sym = symbol_intern_in_package (package, name);
+		      symbol_add_value (sym, posted_data);
 		    }
+
+		  if (name) free (name);
+		  if (filename) free (filename);
+		  if (posted_data) free (posted_data);
 		}
-
-	      /* Find the end of the MIME headers for this document.
-		 I'm not sure what we should do with them.  For now, they
-		 are simply discarded. */
-
-	      while (i < limit)
-		{
-		  while (data[i] != '\n' && data[i] != '\r')
-		    i++;
-
-		  /* Okay, skip one newline. */
-		  if (data[i] == '\r' && data[i + 1] == '\n')
-		    i += 2;
-		  else if (data[i] == '\n')
-		    i += 1;
-
-		  /* If the next character is a newline, skip it also, and
-		     then move on. */
-		  if (data[i] == '\n' || data[i] == '\r')
-		    {
-		      if (data[i] == '\r' && data[i + 1] == '\n')
-			{
-			  i += 2;
-			  break;
-			}
-		      else if (data[i] == '\n')
-			{
-			  i += 1;
-			  break;
-			}
-		    }
-		}
-
-	      /* Find next boundary string.  From here up to that point
-		 is the posted data. */
-	      start = i;
-	      while (i < limit)
-		{
-		  if (strncmp (data + i, boundary, blen) == 0)
-		    break;
-		  i++;
-		}
-
-	      /* Move backwards to the beginning of the line.
-		 More and more, each and every day, do I lower my
-		 opinion of the coders at Netscape. */
-	      if (i < limit)
-		{
-		  while (!word_separator (data[i])) i--;
-
-		  /* Handle CRLF here also.  This couldn't be more
-		     disgusting. */
-		  if (data[i] == '\n' && data[i - 1] == '\r')
-		    i--;
-		}
-
-	      value_len = i - start;
-	      value = (char *)xmalloc (1 + value_len);
-	      memcpy (value, data + start, value_len);
-	      value[value_len] = '\0';
-
-	      /* Save value and name in the posted package. */
-	      if (filename != (char *)NULL)
-		{
-		  Symbol *sym;
-		  Datablock *block = datablock_create (value, value_len);
-
-		  /* Kill old definition if there is one. */
-		  sym = symbol_remove_in_package (package, name);
-		  symbol_free (sym);
-
-		  /* Create new definition. */
-		  sym = symbol_intern_in_package (package, name);
-		  sym->type = symtype_BINARY;
-		  sym->values = (char **)block;
-
-		  /* Create a new variable called NAME-FILENAME. */
-		  {
-		    char *newname = (char *)xmalloc (40 + strlen (name));
-		    sprintf (newname, "POSTED::%s-FILENAME", name);
-		    pagefunc_set_variable (newname, filename);
-		    free (newname);
-		  }
-		}
-	      else
-		{
-		  /* Just a regular variable.  Add this name/value to the
-		     POSTED package. */
-		  char *newname = (char *)xmalloc (20 + strlen (name));
-		  Symbol *sym;
-
-		  sprintf (newname, "POSTED::%s", name);
-		  sym = symbol_intern (newname);
-		  symbol_add_value (sym, value);
-		  free (newname);
-		}
-
-	      if (name) free (name);
-	      if (filename) free (filename);
-	      if (value) free (value);
+		symbol_destroy_package (temp_pack);
+	      }
 	    }
 	  else
 	    i++;
 	}
       free (boundary);
     }
+
+  /* Cause the material in the POSTED package to appear in the DEFAULT
+     package as well.  We pass an empty string because there isn't anything
+     new to add, we just want to copy the data that we just made. */
   mhtml_post_to_document ("");
 }
 
@@ -1838,7 +1892,16 @@ mhttpd_metahtml_engine (HTTP_RESULT *result)
 	 This means that the contents of a binary file is sent over
 	 the net without any encoding at all.  Don't ask me why, I
 	 just work here. */
-      pagefunc_set_variable ("MHTML::RAW-CONTENT-DATA", result->spec->content);
+      {
+	Datablock *block = datablock_create
+	  (result->spec->content, result->spec->content_length);
+	Symbol *sym = symbol_remove ("MHTML::RAW-CONTENT-DATA");
+
+	symbol_free (sym);
+	sym = symbol_intern ("MHTML::RAW-CONTENT-DATA");
+	sym->type = symtype_BINARY;
+	sym->values = (char **)block;
+      }
 
       if (!enctype ||
 	  (strncasecmp (enctype, "multipart/form-data;", 20) != 0))
