@@ -62,7 +62,7 @@ static int mdb_allow_redo = 1;
 static void initialize_page_handlers (void);
 static void parse_file_and_line (char *string, char **file,
 				 int *line, int *defaulted_p);
-
+static void find_file_and_line_for_function (char *, char **, int *);
 static char *mdb_quit (MDBArgs);
 static char *mdb_eval (MDBArgs);
 static char *mdb_help (MDBArgs);
@@ -74,6 +74,7 @@ static char *mdb_breakpoint (MDBArgs);
 static char *mdb_delete (MDBArgs);
 static char *mdb_cont (MDBArgs);
 static char *mdb_next (MDBArgs);
+static char *mdb_print (MDBArgs);
 
 MDBCommand mdb_command_table[] = {
   { "quit",	(char *)NULL,	mdb_quit,	"quit",
@@ -82,6 +83,9 @@ MDBCommand mdb_command_table[] = {
   { "eval",	(char *)NULL,	mdb_eval,	"eval EXPR",
       "Evaluate EXPR in the current context" },
 
+  { "print",	(char *)NULL,	mdb_print,	"print VARIABLE",
+      "Display the contents of VARIABLE in the current context" },
+
   { "help",	"?",		mdb_help,	"help [COMMAND]",
       "Provide help on using MDB" },
 
@@ -182,6 +186,106 @@ make_message (char *format, ...)
   return (result);
 }
 
+typedef struct
+{
+  MDB_File *file;
+  int lineno;
+  char *fname;
+} MDB_FunctionDef;
+
+static MDB_FunctionDef **snarfed_functions = (MDB_FunctionDef **)NULL;
+static int snarfed_slots = 0;
+static int snarfed_index = 0;
+
+static void
+mdb_scan_for_defuns (MDB_File *file, PAGE *page)
+{
+
+  if (page != (PAGE *)NULL)
+    {
+      register int i = 0;
+      register int lineno = 1;
+
+      while  (i < (page->bindex - 4))
+	{
+	  if (page->buffer[i] == '\n')
+	    {
+	      lineno++; i++;
+	      continue;
+	    }
+
+	  if (page->buffer[i] != '<')
+	    {
+	      i++;
+	      continue;
+	    }
+
+	  if ((strncasecmp ((page->buffer + i), "<defun", 6) == 0) ||
+	      (strncasecmp ((page->buffer + i), "<define-", 8) == 0) ||
+	      (strncasecmp ((page->buffer + i), "<defmacro", 9) == 0) ||
+	      (strncasecmp ((page->buffer + i), "<defsubst", 9) == 0))
+	    {
+	      int thisline = lineno + 1;
+	      int start, end;
+	      char *fname = (char *)NULL;
+	      MDB_FunctionDef *fdef = (MDB_FunctionDef *)
+		xmalloc (sizeof (MDB_FunctionDef *));
+
+	      while (!whitespace (page->buffer[i])) i++;
+	      while (whitespace (page->buffer[i]))
+		{
+		  if (page->buffer[i] == '\n')
+		    lineno++;
+		  i++;
+		}
+	      start = i;
+	      while (!whitespace (page->buffer[i])) i++;
+	      end = i;
+
+	      fname = (char *)xmalloc (1 + (end - start));
+	      strncpy (fname, page->buffer + start, end - start);
+	      fname[end - start] = '\0';
+
+	      fdef->file = file;
+	      fdef->lineno = thisline;
+	      fdef->fname = fname;
+
+	      /* Find out if we should replace an existing definition. */
+	      {
+		register int j;
+		MDB_FunctionDef *installed = (MDB_FunctionDef *)NULL;
+
+		for (j = 0; j < snarfed_index; j++)
+		  if (strcasecmp (fname, snarfed_functions[j]->fname) == 0)
+		    {
+		      installed = snarfed_functions[j];
+		      break;
+		    }
+
+		if (installed)
+		  {
+		    free (installed->fname);
+		    free (installed);
+		    snarfed_functions[j] = fdef;
+		  }
+		else
+		  {
+		    if ((snarfed_index + 2) > snarfed_slots)
+		      snarfed_functions = (MDB_FunctionDef **)xrealloc
+			(snarfed_functions,
+			 (snarfed_slots += 20) * sizeof (MDB_FunctionDef *));
+
+		    snarfed_functions[snarfed_index++] = fdef;
+		    snarfed_functions[snarfed_index] = (MDB_FunctionDef *)NULL;
+		  }
+	      }
+	    }
+	  else
+	    i++;
+	}
+    }
+}
+
 static MDB_File *
 mdb_add_file (char *filename, PAGE *page)
 {
@@ -206,6 +310,7 @@ mdb_add_file (char *filename, PAGE *page)
   mdb_files[mdb_files_index++] = file;
   mdb_files[mdb_files_index] = (MDB_File *)NULL;
 
+  mdb_scan_for_defuns (file, page);
   return (file);
 }
 
@@ -439,6 +544,49 @@ mdb_eval (MDBArgs)
   return (result);
 }
 
+static char *
+mdb_print (MDBArgs)
+{
+  PAGE *page = page_create_page ();
+  char *result = (char *)NULL;
+  register int i;
+
+  CANNOT_REDO;
+
+  page_debug_clear ();
+  page_syserr_clear ();
+
+  for (i = 0; line[i] && line[i] != '<'; i++);
+  if (line[i])
+    bprintf (page, "%s", line);
+  else
+    bprintf (page, "<get-var %s>\n", line);
+
+  page_debug_clear ();
+  page_syserr_clear ();
+  page_process_page (page);
+
+  if (page)
+    {
+      result = page->buffer;
+      free (page);
+    }
+
+  if (page_debug_buffer ())
+    {
+      fprintf (stderr, "[DEBUGGING-OUTPUT]: %s\n", page_debug_buffer ());
+      page_debug_clear ();
+    }
+
+  if (page_syserr_buffer ())
+    {
+      fprintf (stderr, "[SYSTEM-ERROR-OUTPUT]: %s\n", page_syserr_buffer ());
+      page_syserr_clear ();
+    }
+
+  return (result);
+}
+
 static char *
 mdb_quit (MDBArgs)
 {
@@ -492,7 +640,7 @@ mdb_exec (MDBArgs)
   if (defaulted)
     {
       if (!last_executed_filename)
-	last_executed_filename = strdup (filename);
+	last_executed_filename = strdup (filename ? filename : "");
       else
 	{
 	  free (filename);
@@ -503,7 +651,7 @@ mdb_exec (MDBArgs)
     {
       if (last_executed_filename)
 	free (last_executed_filename);
-      last_executed_filename = strdup (filename);
+      last_executed_filename = strdup (filename ? filename : "");
     }
 
   file = mdb_find_file (filename);
@@ -566,8 +714,6 @@ static void
 parse_file_and_line (char *string, char **file, int *line, int *defaulted_p)
 {
   register int i;
-  char *colon = strchr (string, ':');
-  char *parseloc = string;
   int digits_only = 1;
   int defaulted = 1;
 
@@ -576,8 +722,16 @@ parse_file_and_line (char *string, char **file, int *line, int *defaulted_p)
 
   if (string && *string)
     {
-      if (colon)
-	parseloc = colon + 1;
+      char *colon = strchr (string, ':');
+      char *parseloc = colon ? colon + 1 : string;
+
+      if ((colon != (char *)NULL) && (*parseloc == ':'))
+	{
+	  /* This isn't a filename, it's a variable name with a package
+	     qualifier!  Don't pretend that it is something different. */
+	  colon = (char *)NULL;
+	  parseloc = string;
+	}
 
       /* Check to see if there are only digits here. */
       for (i = 0; parseloc[i]; i++)
@@ -715,6 +869,17 @@ mdb_list (MDBArgs)
   /* Find the file that the user specified. */
   file = mdb_find_file (filename);
 
+  if (!file)
+    {
+      /* Try to find the function that the user specified. */
+      char *fname = filename;
+      char *found = (char *)NULL;
+      find_file_and_line_for_function (fname, &found, &line_number);
+
+      if (found)
+	file = mdb_find_file (found);
+    }
+
   if (!file)
     return (make_error ("Cannot find `%s' for `%s'", filename, name));
 
@@ -844,7 +1009,7 @@ pf_break_handler (PFunArgs)
 	  printf ("%s", listing);
 	  free (listing);
 	}
-      else
+      /* else */
 	{
 	  /* Print the remainder of the breakpoint here. */
 	  PAGE *source = bp->file->contents;
@@ -876,12 +1041,12 @@ pf_break_handler (PFunArgs)
 static char *
 mdb_cont (MDBArgs)
 {
-  int count = atoi (line);
+  int count = atoi ((line && *line) ? line : "1");
   char *result = (char *)NULL;
 
   if (mdb_loop_level > 1)
     {
-      MDB_ContFlag += count ? count : 1;
+      MDB_ContFlag += count;
       result = strdup ("Continuing...");
     }
 
@@ -892,14 +1057,20 @@ mdb_cont (MDBArgs)
 static char *
 mdb_next (MDBArgs)
 {
-  int count = atoi (line);
+  int count = atoi ((line && *line) ? line : "1");
+  char *result = (char *)NULL;
 
   if (mdb_loop_level > 1)
     {
-      mdb_set_next_breakpoint (mdb_current_bp);
-      MDB_ContFlag += count ? count : 1;
+      if (!mdb_current_bp)
+	result = make_message ("Can't do NEXT from this type of breakpoint!");
+      else
+	{
+	  mdb_set_next_breakpoint (mdb_current_bp);
+	  MDB_ContFlag += count;
+	}
     }
-  return (char *)NULL;
+  return (result);
 }
 
 /* Remove a breakpoint. */
@@ -918,6 +1089,19 @@ mdb_delete (MDBArgs)
     result = make_error ("There is no breakpoint `%s'", line);
   else
     {
+      if ((bps[which]->type == break_CALLBACK) && (bps[which]->fname))
+	{
+	  char *fname = bps[which]->fname;
+	  UserFunction *uf = mhtml_find_user_function (fname);
+	  PFunDesc *desc = pagefunc_get_descriptor (fname);
+
+	  /* Turn off debugging on this function. */
+	  if (uf)
+	    uf->debug_level = 0;
+
+	  if (desc)
+	    desc->debug_level = 0;
+	}
       bps[which]->type = break_DELETED;
       result = make_message ("Deleted breakpoint %d", which + 1);
     }
@@ -938,35 +1122,160 @@ initialize_page_handlers (void)
   sym = symbol_intern_in_package 
     (mhtml_function_package, MDB_BreakPFunDesc.tag);
   sym->values = (char **)&MDB_BreakPFunDesc;
+  sym->type = symtype_FUNCTION;
 
   sym = symbol_intern_in_package (mhtml_function_package, "INCLUDE");
   sym->values = (char **)&MDB_IncludePFunDesc;
+  sym->type = symtype_FUNCTION;
+}
+
+/* Find the file and line-number of the function FUNCTION if possible. */
+static void
+find_file_and_line_for_function (char *function, char **filename, int *line)
+{
+  register int i;
+  MDB_FunctionDef *fdef = (MDB_FunctionDef *)NULL;
+
+  for (i = 0; i < snarfed_index; i++)
+    {
+      if (strcasecmp (snarfed_functions[i]->fname, function) == 0)
+	{
+	  fdef = snarfed_functions[i];
+	  break;
+	}
+    }
+
+  if (fdef)
+    {
+      *filename = fdef->file->filename;
+      *line = fdef->lineno;
+    }
+}
+
+static int
+mdb_callback (PFunArgs, PFunDesc *desc, UserFunction *uf, char *open_body)
+{
+  char *fname = desc->tag;
+  char *ftype;
+  char *arg_string;
+  int which_breakpoint = -1;
+  /* MDB_Breakpoint *bp = (MDB_Breakpoint *)NULL; */
+
+  if (uf != (UserFunction *)NULL)
+    {
+      fname = uf->name;
+      switch (uf->type)
+	{
+	case user_MACRO:
+	  ftype = "defmacro";
+	  if (uf->flags & user_WEAK_MACRO)
+	    ftype = "defweakmacro";
+	  break;
+	case user_SUBST: ftype = "defsubst"; break;
+	case user_DEFUN: ftype = "defun"; break;
+	default: ftype = "BADCODE (1)";
+	}
+    }
+  else
+    {
+      fname = desc->tag;
+      switch (desc->complexp)
+	{
+	case 0:  ftype = "internal defun"; break;
+	case 1:  ftype = "internal defmacro"; break;
+	case -1: ftype = "internal defweakmacro"; break;
+	default: ftype = "internal BADCODE (2)";
+	}
+    }
+
+  which_breakpoint = mdb_find_breakpoint_function (fname);
+
+  arg_string = mhtml_funargs (vars);
+  printf ("\n*Bpt %d in %s %s\n", which_breakpoint + 1, ftype, fname);
+  printf ("* <%s %s>\n", fname, arg_string);
+  free (arg_string);
+
+  mdb_loop ();
+
+  /* If the user has quit, under NO CIRCUMSTANCES should we
+     continue to execute the code. */
+  if (MDB_QuitFlag)
+    mdb_throw_to_top_level ();
+
+  return (1);
 }
 
 static char *
 mdb_breakpoint (MDBArgs)
 {
   int line_number, defaulted;
-  char *filename;
+  char *filename = (char *)NULL;
   char *result = (char *)NULL;
-  MDB_File *file;
+  int callback_break = 0;
+  char *fname = (char *)NULL;
 
   CANNOT_REDO;
 
   parse_file_and_line (line, &filename, &line_number, &defaulted);
 
-  if (!*line || line_number < 1)
-    return (make_error ("`%s' requires a line number", name));
+  if (!*line || !defaulted || (line_number < 1))
+    {
+      if (filename != (char *)NULL)
+	find_file_and_line_for_function (filename, &filename, &line_number);
+    }
 
-  file = mdb_find_file (filename);
+  if (!*line || !defaulted || (line_number < 1))
+    {
+      if (filename != (char *)NULL)
+	{
+	  /* Try to set a breakpoint on a function which isn't currently
+	     loaded in any file.  Only can do this if the function is
+	     defined already. */
+	  UserFunction *uf = mhtml_find_user_function (filename);
+	  PFunDesc *desc = pagefunc_get_descriptor (filename);
 
-  if (!file)
-    result = make_error ("Cannot load `%s' for `%s'", name, filename);
+	  if ((uf != (UserFunction *)NULL) || (desc != (PFunDesc *)NULL))
+	    {
+	      /* We can set a callback breakpoint on this function. */
+	      if (uf != (UserFunction *)NULL)
+		uf->debug_level = -2;
+	      else
+		desc->debug_level = -1;
+
+	      fname = strdup (filename);
+	      free (filename);
+	      filename = (char *)xmalloc (20 + strlen (filename));
+	      sprintf (filename, "#<internal-file: %s>", fname);
+
+	      line_number = 0;
+	      callback_break = 1;
+	    }
+	}
+      else
+	return
+	  (make_error
+	   ("`%s' requires a file name, function name, or line number", name));
+    }
+
+  if (callback_break)
+    {
+      mdb_add_breakpoint ((MDB_File *)NULL, &line_number, break_CALLBACK,
+			  fname);
+      result = make_message ("Break in function %s.", fname);
+      mhtml_debug_callback_function = mdb_callback;
+    }
   else
     {
-      mdb_add_breakpoint (file, line_number, break_USER);
-      result = make_message ("Break at line %05d, in file %s.",
-			     line_number, file->nameonly);
+      MDB_File *file = mdb_find_file (filename);
+
+      if (!file)
+	result = make_error ("Cannot load `%s' for `%s'", name, filename);
+      else
+	{
+	  mdb_add_breakpoint (file, &line_number, break_USER, (char *)NULL);
+	  result = make_message ("Break at line %05d, in file %s.",
+				 line_number, file->nameonly);
+	}
     }
   return (result);
 }
