@@ -299,7 +299,11 @@ prepare_terminal_settings (int meta_flag, TIOTYPE otio, TIOTYPE *tiop)
      use it for the meta-key.  If only one of even or odd parity is
      specified, then the terminal is using parity, and we cannot. */
 #if !defined (ANYP)
-#  define ANYP (EVENP | ODDP)
+#  if defined (EVENP) && defined (ODDP)
+#    define ANYP (EVENP | ODDP)
+#  else
+#    define ANYP 0
+#  endif
 #endif
   if (((otio.sgttyb.sg_flags & ANYP) == ANYP) ||
       ((otio.sgttyb.sg_flags & ANYP) == 0))
@@ -354,7 +358,7 @@ prepare_terminal_settings (int meta_flag, TIOTYPE otio, TIOTYPE *tiop)
 #  define VTIME VEOL
 #endif
 
-#if defined (TERMIOS_TTY_DRIVER)
+#if defined (HAVE_TERMIOS_H)
 #  define TIOTYPE struct termios
 #  define DRAIN_OUTPUT(fd)	tcdrain (fd)
 #  define GETATTR(tty, tiop)	(tcgetattr (tty, tiop))
@@ -430,14 +434,14 @@ set_tty_settings (int tty, TIOTYPE *tiop)
 }
 
 static void
-prepare_terminal_settings (int meta_flag, TIOTYPE otio, TIOTYPE *tiop)
+prepare_terminal_settings (int meta_flag, TIOTYPE oldtio, TIOTYPE *tiop)
 {
-  readline_echoing_p = (otio.c_lflag & ECHO);
+  readline_echoing_p = (oldtio.c_lflag & ECHO);
 
   tiop->c_lflag &= ~(ICANON | ECHO);
 
-  if ((unsigned char) otio.c_cc[VEOF] != (unsigned char) _POSIX_VDISABLE)
-    _rl_eof_char = otio.c_cc[VEOF];
+  if ((unsigned char) oldtio.c_cc[VEOF] != (unsigned char) _POSIX_VDISABLE)
+    _rl_eof_char = oldtio.c_cc[VEOF];
 
 #if defined (USE_XON_XOFF)
 #if defined (IXANY)
@@ -468,14 +472,14 @@ prepare_terminal_settings (int meta_flag, TIOTYPE otio, TIOTYPE *tiop)
   if (OUTPUT_BEING_FLUSHED (tiop))
     {
       tiop->c_lflag &= ~FLUSHO;
-      otio.c_lflag &= ~FLUSHO;
+      oldtio.c_lflag &= ~FLUSHO;
     }
 #endif
 
   /* Turn off characters that we need on Posix systems with job control,
      just to be sure.  This includes ^Y and ^V.  This should not really
      be necessary.  */
-#if defined (TERMIOS_TTY_DRIVER) && defined (_POSIX_VDISABLE)
+#if defined (HAVE_TERMIOS_H) && defined (_POSIX_VDISABLE)
 
 #if defined (VLNEXT)
   tiop->c_cc[VLNEXT] = _POSIX_VDISABLE;
@@ -693,3 +697,4 @@ rltty_set_default_bindings (Keymap kmap)
     }
 #endif /* !NEW_TTY_DRIVER */
 }
+
