@@ -51,6 +51,7 @@ static void usage (void);
 
 static char *progname = (char *)NULL;
 static PAGE *input_contents = (PAGE *)NULL;
+static PAGE *config_page = (PAGE *)NULL;
 
 extern char *metahtml_copyright_string;
 
@@ -61,6 +62,8 @@ extern char *metahtml_copyright_string;
 int
 main (int argc, char *argv[])
 {
+  int exit_code = 0;
+
 #if defined (NOTDEF)
   register int i;
 
@@ -75,6 +78,12 @@ main (int argc, char *argv[])
 
   prep_for_page_process (input_contents, argc, argv);
 
+  if (config_page != (PAGE *)NULL)
+    {
+      page_process_page (config_page);
+      if (config_page) page_free_page (config_page);
+    }
+
   page_process_page (input_contents);
   if (input_contents && input_contents->buffer)
     {
@@ -82,7 +91,14 @@ main (int argc, char *argv[])
 	fprintf (stdout, "%s", input_contents->buffer);
     }
 
-  return (0);
+  /* Perhaps return a different exit code. */
+  {
+    char *exit_code_text = pagefunc_get_variable ("mhc::exit-code");
+    if (!empty_string_p (exit_code_text))
+      exit_code = atoi (exit_code_text);
+  }
+
+  return (exit_code);
 }
 
 static void
@@ -152,6 +168,18 @@ parse_program_args (int argc, char *argv[])
 	}
       else if (!filename && (strcmp (arg, "-z") == 0))
 	call_bootstrap = 0;
+      else if (!filename && (strcmp (arg, "--config") == 0))
+	{
+	  char *config_file = argv[arg_index];
+
+	  if (config_file == (char *)NULL)
+	    usage ();
+	  else
+	    {
+	      config_page = page_read_template (config_file);
+	      arg_index++;
+	    }
+	}
       else if (!filename && (*arg != '-'))
 	filename = arg;
       else if (filename)
