@@ -109,10 +109,6 @@ static void initialize_server (void);
 static void setup_signal_handling (void);
 static void fatal (char *format, ...);
 
-#if defined (CHECK_TIMESTAMP)
-static void check_timestamp (char *filename);
-#endif
-
 #if defined (CHECK_ACTIVATION)
 extern int check_activation_key (char *key);
 #endif
@@ -125,10 +121,6 @@ main (int argc, char *argv[])
   progname = argv[0];
   parse_program_args (argc, argv);
 
-#if defined (CHECK_TIMESTAMP)
-  check_timestamp (argv[0]);
-#endif
-
   if (!once_and_exit)
     {
       fprintf (stderr, "%s: Initializing server...", argv[0]); 
@@ -239,25 +231,6 @@ main (int argc, char *argv[])
   exit (0);
 }
 
-#if defined (CHECK_TIMESTAMP)
-#include "../libtimestamp/timestamp.h"
-
-static void
-check_timestamp (char *filename)
-{
-  if (check_stamp_expired (filename))
-    {
-      fprintf (stderr, "Sorry, but this Beta release has expired.\n");
-      fprintf (stderr, "You may obtain another copy of the server from\n");
-      fprintf (stderr, "\n\thttp://www.metahtml.com\n\n");
-      fprintf (stderr, "We hope that you have enjoyed the use of our\n");
-      fprintf (stderr, "server, and will continue to watch our site for\n");
-      fprintf (stderr, "new additions to our product suite.\n");
-      exit (2);
-    }
-}
-#endif /* CHECK_TIMESTAMP */
-
 static char *deletable_variables[] =
 {
   "mhtml::server-name",
@@ -1536,7 +1509,8 @@ loop_over_connections (void)
 	    result = http_handle_request (req, connection);
 
 	  if (result && result->page && result->page->buffer)
-	    mhttpd_write (connection, result->page->buffer, result->page->bindex);
+	    mhttpd_write (connection, result->page->buffer,
+			  result->page->bindex);
 
 #if defined (USE_KEEP_ALIVE)
 	  if ((result != (HTTP_RESULT *)NULL) &&
